@@ -19,6 +19,7 @@ const char Change_fileid[] = "Hatari change.c : " __DATE__ " " __TIME__;
 #include "change.h"
 #include "dialog.h"
 #include "floppy.h"
+#include "fdc.h"
 #include "gemdos.h"
 #include "hdc.h"
 #include "ide.h"
@@ -44,9 +45,9 @@ const char Change_fileid[] = "Hatari change.c : " __DATE__ " " __TIME__;
 
 #define DEBUG 0
 #if DEBUG
-#define Dprintf(a) printf(a)
+#define Dprintf(a...) printf(a)
 #else
-#define Dprintf(a)
+#define Dprintf(a...)
 #endif
 
 /*-----------------------------------------------------------------------*/
@@ -56,6 +57,8 @@ const char Change_fileid[] = "Hatari change.c : " __DATE__ " " __TIME__;
  */
 bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 {
+	int i;
+
 	/* Did we change monitor type? If so, must reset */
 	if (current->Screen.nMonitorType != changed->Screen.nMonitorType
 	    && (changed->System.nMachineType == MACHINE_FALCON
@@ -79,10 +82,13 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 		return true;
 
 	/* Did change ACSI hard disk image? */
-	if (changed->HardDisk.bUseHardDiskImage != current->HardDisk.bUseHardDiskImage
-	    || (strcmp(changed->HardDisk.szHardDiskImage, current->HardDisk.szHardDiskImage)
-	        && changed->HardDisk.bUseHardDiskImage))
-		return true;
+	for (i = 0; i < MAX_ACSI_DEVS; i++)
+	{
+		if (changed->Acsi[i].bUseDevice != current->Acsi[i].bUseDevice
+		    || (strcmp(changed->Acsi[i].sDeviceFile, current->Acsi[i].sDeviceFile)
+		        && changed->Acsi[i].bUseDevice))
+			return true;
+	}
 
 	/* Did change IDE master hard disk image? */
 	if (changed->HardDisk.bUseIdeMasterHardDiskImage != current->HardDisk.bUseIdeMasterHardDiskImage
@@ -236,6 +242,16 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 			bFloppyInsert[i] = false;
 	}
 
+	if ( changed->DiskImage.EnableDriveA != current->DiskImage.EnableDriveA )
+		FDC_Drive_Set_Enable ( 0 , changed->DiskImage.EnableDriveA );
+	if ( changed->DiskImage.EnableDriveB != current->DiskImage.EnableDriveB )
+		FDC_Drive_Set_Enable ( 1 , changed->DiskImage.EnableDriveB );
+
+	if ( changed->DiskImage.DriveA_NumberOfHeads != current->DiskImage.DriveA_NumberOfHeads )
+		FDC_Drive_Set_NumberOfHeads ( 0 , changed->DiskImage.DriveA_NumberOfHeads );
+	if ( changed->DiskImage.DriveB_NumberOfHeads != current->DiskImage.DriveB_NumberOfHeads )
+		FDC_Drive_Set_NumberOfHeads ( 1 , changed->DiskImage.DriveB_NumberOfHeads );
+
 	/* Did change GEMDOS drive? */
 	if (changed->HardDisk.bUseHardDiskDirectories != current->HardDisk.bUseHardDiskDirectories
 	    || (strcmp(changed->HardDisk.szHardDiskDirectories[0], current->HardDisk.szHardDiskDirectories[0])
@@ -246,16 +262,20 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 		bReInitGemdosDrive = true;
 	}
 
-	/* Did change HD image? */
-	if (changed->HardDisk.bUseHardDiskImage != current->HardDisk.bUseHardDiskImage
-	    || (strcmp(changed->HardDisk.szHardDiskImage, current->HardDisk.szHardDiskImage)
-	        && changed->HardDisk.bUseHardDiskImage))
+	/* Did change ACSI image? */
+	for (i = 0; i < MAX_ACSI_DEVS; i++)
 	{
-		Dprintf("- HD image>\n");
-		HDC_UnInit();
-		bReInitAcsiEmu = true;
+		if (changed->Acsi[i].bUseDevice != current->Acsi[i].bUseDevice
+		    || (strcmp(changed->Acsi[i].sDeviceFile, current->Acsi[i].sDeviceFile)
+		        && changed->Acsi[i].bUseDevice))
+		{
+			Dprintf("- ACSI image %i>\n", i);
+			bReInitAcsiEmu = true;
+		}
 	}
-	
+	if (bReInitAcsiEmu)
+		HDC_UnInit();
+
 	/* Did change IDE HD master image? */
 	if (changed->HardDisk.bUseIdeMasterHardDiskImage != current->HardDisk.bUseIdeMasterHardDiskImage
 	    || (strcmp(changed->HardDisk.szIdeMasterHardDiskImage, current->HardDisk.szIdeMasterHardDiskImage)
@@ -338,7 +358,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	}
 
 	/* Mount a new HD image: */
-	if (bReInitAcsiEmu && ConfigureParams.HardDisk.bUseHardDiskImage)
+	if (bReInitAcsiEmu)
 	{
 		Dprintf("- HD<\n");
 		HDC_Init();
@@ -482,7 +502,7 @@ bool Change_ApplyCommandline(char *cmdline)
 	inarg = argc = 0;
 	for (i = 0; cmdline[i]; i++)
 	{
-		if (isspace(cmdline[i]) && cmdline[i-1] != '\\')
+		if (isspace((unsigned char)cmdline[i]) && cmdline[i-1] != '\\')
 		{
 			inarg = 0;
 			continue;
@@ -511,7 +531,7 @@ bool Change_ApplyCommandline(char *cmdline)
 	argv[argc++] = "hatari";
 	for (i = 0; cmdline[i]; i++)
 	{
-		if (isspace(cmdline[i]))
+		if (isspace((unsigned char)cmdline[i]))
 		{
 			if (cmdline[i-1] != '\\')
 			{
