@@ -19,6 +19,7 @@ const char Control_fileid[] = "Hatari control.c : " __DATE__ " " __TIME__;
 #include <sys/time.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <assert.h>
 
 #include "main.h"
 #include "change.h"
@@ -94,7 +95,7 @@ static bool Control_InsertKey(const char *event)
 			IKBD_PressSTKey(keycode, false);
 		}
 	} else {
-		if (!isalnum(key[0])) {
+		if (!isalnum((unsigned char)key[0])) {
 			fprintf(stderr, "ERROR: non-alphanumeric character '%c' needs to be given as keycode\n", key[0]);
 			return false;
 		}
@@ -270,7 +271,7 @@ static bool Control_SetPath(char *name)
  */
 static bool Control_Usage(const char *cmd)
 {
-	fprintf(stderr, "ERROR: unrecognized hatari command: '%s'", cmd);
+	fprintf(stderr, "ERROR: unrecognized hatari command: '%s'!\n", cmd);
 	fprintf(stderr,
 		"Supported commands are:\n"
 		"- hatari-debug <Debug UI command>\n"
@@ -292,13 +293,19 @@ static bool Control_Usage(const char *cmd)
 /*-----------------------------------------------------------------------*/
 /**
  * Parse Hatari debug/event/option/toggle/path/shortcut command buffer.
- * Given buffer is modified in-place.
  */
-void Control_ProcessBuffer(char *buffer)
+void Control_ProcessBuffer(const char *orig)
 {
-	char *cmd, *cmdend, *arg;
+	char *cmd, *cmdend, *arg, *buffer;
 	int ok = true;
-	
+
+	/* this is called from several different places,
+	 * so take a copy of the original buffer so
+	 * that it can be sliced & diced
+	 */
+	buffer = strdup(orig);
+	assert(buffer);
+
 	cmd = buffer;
 	do {
 		/* command terminator? */
@@ -350,6 +357,7 @@ void Control_ProcessBuffer(char *buffer)
 			cmd = cmdend + 1;
 		}
 	} while (ok && cmdend && *cmd);
+	free(buffer);
 }
 
 
