@@ -28,11 +28,9 @@ typedef enum {
 	REASON_USER        // e.g. keyboard shortcut
 } debug_reason_t;
 
-/* Whether CPU exceptions invoke DebugUI */
-extern int bExceptionDebugging;
-
 extern void DebugUI_Init(void);
 extern void DebugUI(debug_reason_t reason);
+extern void DebugUI_Exceptions(int nr, long pc);
 extern bool DebugUI_ParseLine(const char *input);
 extern bool DebugUI_SetParseFile(const char *input);
 extern void DebugUI_MemorySnapShot_Capture(const char *path, bool bSave);
