@@ -13,7 +13,7 @@ typedef enum {
 	HISTORY_TRACK_NONE = 0,
 	HISTORY_TRACK_CPU = 1,
 	HISTORY_TRACK_DSP = 2,
-	HISTORY_TRACK_ALL = 3,
+	HISTORY_TRACK_ALL = 3
 } history_type_t;
 
 extern history_type_t HistoryTracking;
@@ -36,6 +36,7 @@ extern void History_Show(Uint32 count);
 
 /* for debugui */
 extern void History_Mark(debug_reason_t reason);
+extern char *History_Match(const char *text, int state);
 extern int History_Parse(int nArgc, char *psArgv[]);
 
 #endif
