@@ -11,6 +11,39 @@
 #include <SDL_types.h>
 
 
+/* Exception debugging
+ * -------------------
+ */
+
+/* CPU exception flags
+ * is catching needed also for: traps 0, 3-12, 15? (MonST catches them)
+ */
+#define	EXCEPT_BUS	 (1<<0)
+#define	EXCEPT_ADDRESS 	 (1<<1)
+#define	EXCEPT_ILLEGAL	 (1<<2)
+#define	EXCEPT_ZERODIV	 (1<<3)
+#define	EXCEPT_CHK	 (1<<4)
+#define	EXCEPT_TRAPV	 (1<<5)
+#define	EXCEPT_PRIVILEGE (1<<6)
+#define	EXCEPT_NOHANDLER (1<<7)
+
+/* DSP exception flags */
+#define EXCEPT_DSP	 (1<<8)
+
+/* whether to enable exception debugging on autostart */
+#define EXCEPT_AUTOSTART (1<<9)
+
+/* general flags */
+#define	EXCEPT_NONE	 (0)
+#define	EXCEPT_ALL	 (~EXCEPT_AUTOSTART)
+
+/* defaults are same as with earlier -D option */
+#define DEFAULT_EXCEPTIONS (EXCEPT_BUS|EXCEPT_ADDRESS|EXCEPT_DSP)
+
+extern int ExceptionDebugMask;
+extern const char* Log_SetExceptionDebugMask(const char *OptionsStr);
+
+
 /* Logging
  * -------
  * Is always enabled as it's information that can be useful
@@ -126,6 +159,10 @@ extern char *Log_MatchTrace(const char *text, int state);
 
 #define TRACE_NVRAM		 (1ll<<45)
 
+#define TRACE_SCSI_CMD		 (1ll<<46)
+
+#define TRACE_NATFEATS		 (1ll<<47)
+
 #define	TRACE_NONE		 (0)
 #define	TRACE_ALL		 (~0)
 
