@@ -8,6 +8,19 @@
 #ifndef HATARI_PROFILE_H
 #define HATARI_PROFILE_H
 
+/* caller types */
+#define CALL_UNDEFINED	0	/* = call type information not supported */
+typedef enum {
+	CALL_UNKNOWN	= 1,
+	CALL_NEXT	= 2,
+	CALL_BRANCH	= 4,
+	CALL_SUBROUTINE	= 8,
+	CALL_SUBRETURN	= 16,
+	CALL_EXCEPTION	= 32,
+	CALL_EXCRETURN	= 64,
+	CALL_INTERRUPT	= 128
+} calltype_t;
+
 /* profile command parsing */
 extern const char Profile_Description[];
 extern char *Profile_Match(const char *text, int state);
