@@ -1,7 +1,7 @@
 /*
  * Hatari - symbols.c
  * 
- * Copyright (C) 2010-2013 by Eero Tamminen
+ * Copyright (C) 2010-2014 by Eero Tamminen
  * 
  * This file is distributed under the GNU General Public License, version 2
  * or at your option any later version. Read the file gpl.txt for details.
@@ -31,7 +31,6 @@ const char Symbols_fileid[] = "Hatari symbols.c : " __DATE__ " " __TIME__;
 #include "debug_priv.h"
 #include "debugInfo.h"
 #include "evaluate.h"
-#include "gemdos.h"
 
 typedef struct {
 	char *name;
@@ -62,6 +61,11 @@ typedef struct {
 static symbol_list_t *CpuSymbolsList;
 static symbol_list_t *DspSymbolsList;
 
+/* path for last loaded program (through GEMDOS HD emulation) */
+static char *CurrentProgramPath;
+static bool SymbolsAreForProgram;
+static bool AutoLoadFailed;
+
 
 /* ------------------ load and free functions ------------------ */
 
@@ -137,6 +141,21 @@ static void symbol_list_free(symbol_list_t *list)
 	}
 }
 
+/**
+ * Return symbol type identifier char
+ */
+static char symbol_char(int type)
+{
+	switch (type) {
+	case SYMTYPE_TEXT: return 'T';
+	case SYMTYPE_DATA: return 'D';
+	case SYMTYPE_BSS:  return 'B';
+	default: return '?';
+	}
+}
+
+#define INVALID_SYMBOL_OFFSETS ((symbol_list_t*)1)
+
 /**
  * Load symbols of given type and the symbol address addresses from
  * DRI/GST format symbol table, and add given offsets to the addresses:
@@ -145,7 +164,7 @@ static void symbol_list_free(symbol_list_t *list)
  */
 static symbol_list_t* symbols_load_dri(FILE *fp, prg_section_t *sections, symtype_t gettype, Uint32 tablesize)
 {
-	int i, count, symbols, len;
+	int i, count, symbols, len, outside;
 	int dtypes, locals, ofiles;
 	prg_section_t *section;
 	symbol_list_t *list;
@@ -155,7 +174,7 @@ static symbol_list_t* symbols_load_dri(FILE *fp, prg_section_t *sections, symtyp
 	Uint16 symid;
 	Uint32 address;
 
-	if (tablesize % DRI_ENTRY_SIZE) {
+	if (tablesize % DRI_ENTRY_SIZE || !tablesize) {
 		fprintf(stderr, "ERROR: invalid DRI/GST symbol table size %d!\n", tablesize);
 		return NULL;
 	}
@@ -164,7 +183,7 @@ static symbol_list_t* symbols_load_dri(FILE *fp, prg_section_t *sections, symtyp
 		return NULL;
 	}
 
-	dtypes = ofiles = locals = count = 0;
+	outside = dtypes = ofiles = locals = count = 0;
 	for (i = 1; i <= symbols; i++) {
 		/* read DRI symbol table slot */
 		if (fread(name, 8, 1, fp) != 1 ||
@@ -223,7 +242,15 @@ static symbol_list_t* symbols_load_dri(FILE *fp, prg_section_t *sections, symtyp
 		}
 		address += section->offset;
 		if (address > section->end) {
-			fprintf(stderr, "WARNING: ignoring symbol '%s' in slot %d with invalid offset 0x%x (>= 0x%x).\n", name, i, address, section->end);
+			/* VBCC has 1 symbol outside of its section */
+			if (++outside > 2) {
+				/* potentially buggy version of VBCC vlink used */
+				fprintf(stderr, "ERROR: too many invalid offsets, skipping rest of symbols!\n");
+				symbol_list_free(list);
+				return INVALID_SYMBOL_OFFSETS;
+			}
+			fprintf(stderr, "WARNING: ignoring symbol '%s' of %c type in slot %d with invalid offset 0x%x (>= 0x%x).\n",
+				name, symbol_char(symtype), i, address, section->end);
 			continue;
 		}
 		list->names[count].address = address;
@@ -263,11 +290,15 @@ static symbol_list_t* symbols_load_dri(FILE *fp, prg_section_t *sections, symtyp
  */
 static symbol_list_t* symbols_load_binary(FILE *fp, symtype_t gettype)
 {
-	Uint32 textlen, datalen, bsslen, start, tablesize, tabletype;
+	Uint32 textlen, datalen, bsslen, start, tablesize, tabletype, prgflags;
 	prg_section_t sections[3];
 	int offset, reads = 0;
+	Uint16 relocflag;
+	const char *info;
+	symbol_list_t* symbols;
 
 	/* get TEXT, DATA & BSS section sizes */
+	fseek(fp, 2, SEEK_SET);
 	reads += fread(&textlen, sizeof(textlen), 1, fp);
 	textlen = SDL_SwapBE32(textlen);
 	reads += fread(&datalen, sizeof(datalen), 1, fp);
@@ -284,17 +315,18 @@ static symbol_list_t* symbols_load_binary(FILE *fp, symtype_t gettype)
 	}
 	reads += fread(&tabletype, sizeof(tabletype), 1, fp);
 	tabletype = SDL_SwapBE32(tabletype);
-	if (reads != 5) {
+
+	/* get program header and whether there's reloc table */
+	reads += fread(&prgflags, sizeof(prgflags), 1, fp);
+	prgflags = SDL_SwapBE32(prgflags);
+	reads += fread(&relocflag, sizeof(relocflag), 1, fp);
+	relocflag = SDL_SwapBE32(relocflag);
+	
+	if (reads != 7) {
 		fprintf(stderr, "ERROR: program header reading failed!\n");
 		return NULL;
 	}
 
-	/* go to start of symbol table */
-	offset = 0x1C + textlen + datalen;
-	if (fseek(fp, offset, SEEK_SET) < 0) {
-		perror("ERROR: seeking to symbol table failed");
-		return NULL;
-	}
 	/* offsets & max sizes for running program TEXT/DATA/BSS section symbols */
 	start = DebugInfo_GetTEXT();
 	if (!start) {
@@ -309,24 +341,47 @@ static symbol_list_t* symbols_load_binary(FILE *fp, symtype_t gettype)
 	}
 
 	start = DebugInfo_GetDATA();
-	sections[1].offset = start - textlen;
+	sections[1].offset = start;
 	sections[1].end = start + datalen - 1;
 
 	start = DebugInfo_GetBSS();
-	sections[2].offset = start - textlen - datalen;
+	sections[2].offset = start;
 	sections[2].end = start + bsslen - 1;
 
+	/* go to start of symbol table */
+	offset = 0x1C + textlen + datalen;
+	if (fseek(fp, offset, SEEK_SET) < 0) {
+		perror("ERROR: seeking to symbol table failed");
+		return NULL;
+	}
 	switch (tabletype) {
 	case 0x4D694E54:	/* "MiNT" */
-		fprintf(stderr, "MiNT executable, trying to load GST symbol table at offset 0x%x...\n", offset);
-		return symbols_load_dri(fp, sections, gettype, tablesize);
+		info = "GCC/MiNT executable, GST symbol table.";
+		break;
 	case 0x0:
-		fprintf(stderr, "Old style excutable, loading DRI / GST symbol table at offset 0x%x.\n", offset);
-		return symbols_load_dri(fp, sections, gettype, tablesize);
+		info = "TOS executable, DRI / GST symbol table.";
+		break;
 	default:
 		fprintf(stderr, "ERROR: unknown executable type 0x%x at offset 0x%x!\n", tabletype, offset);
+		return NULL;
 	}
-	return NULL;
+	fprintf(stderr, "0x%x program flags, reloc=%d, %s\n", prgflags, relocflag, info);
+	fprintf(stderr, "Trying to load symbol table at offset 0x%x...\n", offset);
+	symbols = symbols_load_dri(fp, sections, gettype, tablesize);
+
+	if (symbols == INVALID_SYMBOL_OFFSETS && fseek(fp, offset, SEEK_SET) == 0) {
+		fprintf(stderr, "Re-trying with TEXT-relative BSS/DATA section offsets...\n");
+		start = DebugInfo_GetTEXT();
+		sections[1].offset = start;
+		sections[2].offset = start;
+		sections[1].end += textlen;
+		sections[2].end += (textlen + datalen);
+		symbols = symbols_load_dri(fp, sections, gettype, tablesize);
+	}
+	if (symbols == INVALID_SYMBOL_OFFSETS) {
+		return NULL;
+	}
+	return symbols;
 }
 
 /**
@@ -350,12 +405,16 @@ static symbol_list_t* symbols_load_ascii(FILE *fp, Uint32 *offsets, Uint32 maxad
 			continue;
 		}
 		/* skip empty lines */
-		for (buf = buffer; isspace(*buf); buf++);
+		for (buf = buffer; isspace((unsigned char)*buf); buf++);
 		if (!*buf) {
 			continue;
 		}
 		symbols++;
 	}
+	if (!symbols) {
+		fprintf(stderr, "ERROR: no symbols.\n");
+	}
+
 	fseek(fp, 0, SEEK_SET);
 
 	/* allocate space for symbol list & names */
@@ -371,7 +430,7 @@ static symbol_list_t* symbols_load_ascii(FILE *fp, Uint32 *offsets, Uint32 maxad
 			continue;
 		}
 		/* skip empty lines */
-		for (buf = buffer; isspace(*buf); buf++);
+		for (buf = buffer; isspace((unsigned char)*buf); buf++);
 		if (!*buf) {
 			continue;
 		}
@@ -380,7 +439,7 @@ static symbol_list_t* symbols_load_ascii(FILE *fp, Uint32 *offsets, Uint32 maxad
 			fprintf(stderr, "WARNING: syntax error on line %d, skipping.\n", line);
 			continue;
 		}
-		switch (toupper(symchar)) {
+		switch (toupper((unsigned char)symchar)) {
 		case 'T':
 			symtype = SYMTYPE_TEXT;
 			offset = offsets[0];
@@ -417,6 +476,23 @@ static symbol_list_t* symbols_load_ascii(FILE *fp, Uint32 *offsets, Uint32 maxad
 	return list;
 }
 
+/**
+ * Return true if given FILE* is Atari program.
+ */
+static bool is_atari_program(FILE *fp)
+{
+	long oldpos = ftell(fp);
+	Uint16 magic;
+
+	fseek(fp, 0, SEEK_SET);
+	if (fread(&magic, sizeof(magic), 1, fp) != 1) {
+		return false;
+	}
+	fseek(fp, oldpos, SEEK_SET);
+
+	return (SDL_SwapBE16(magic) == 0x601A);
+}
+
 /**
  * Load symbols of given type and the symbol address addresses from
  * the given file and add given offsets to the addresses.
@@ -425,34 +501,32 @@ static symbol_list_t* symbols_load_ascii(FILE *fp, Uint32 *offsets, Uint32 maxad
 static symbol_list_t* Symbols_Load(const char *filename, Uint32 *offsets, Uint32 maxaddr)
 {
 	symbol_list_t *list;
-	Uint16 magic;
 	FILE *fp;
 
 	if (!(fp = fopen(filename, "r"))) {
 		fprintf(stderr, "ERROR: opening '%s' failed!\n", filename);
 		return NULL;
 	}
-	if (fread(&magic, sizeof(magic), 1, fp) != 1) {
-		fprintf(stderr, "ERROR: reading file '%s' failed.\n", filename);
-		fclose(fp);
-		return NULL;
-	}
-
-	if (SDL_SwapBE16(magic) == 0x601A) {
-		const char *last = GemDOS_GetLastProgramPath();
-		fprintf(stderr, "Reading symbols from program '%s' symbol table...\n", filename);
-		if (strcmp(last, filename) != 0) {
-			fprintf(stderr, "WARNING: given program doesn't match last program executed by GEMDOS HD emulation:\n\t%s", last);
+	if (is_atari_program(fp)) {
+		const char *last = CurrentProgramPath;
+		if (!last) {
+			/* "pc=text" breakpoint used as point for loading program symbols gives false hits during bootup */
+			fprintf(stderr, "WARNING: no program loaded yet (through GEMDOS HD emu)!\n");
+		} else if (strcmp(last, filename) != 0) {
+			fprintf(stderr, "WARNING: given program doesn't match last program executed by GEMDOS HD emulation:\n\t%s\n", last);
 		}
+		fprintf(stderr, "Reading symbols from program '%s' symbol table...\n", filename);
 		list = symbols_load_binary(fp, SYMTYPE_ALL);
+		SymbolsAreForProgram = true;
 	} else {
 		fprintf(stderr, "Reading 'nm' style ASCII symbols from '%s'...\n", filename);
 		list = symbols_load_ascii(fp, offsets, maxaddr, SYMTYPE_ALL);
+		SymbolsAreForProgram = false;
 	}
 	fclose(fp);
 
 	if (!list) {
-		fprintf(stderr, "ERROR: no symbols, or reading them from '%s' failed!\n", filename);
+		fprintf(stderr, "ERROR: reading symbols from '%s' failed!\n", filename);
 		return NULL;
 	}
 
@@ -735,7 +809,7 @@ int Symbols_DspCount(void)
 	return (DspSymbolsList ? DspSymbolsList->count : 0);
 }
 
-/* ---------------- symbol showing and command parsing ------------------ */
+/* ---------------- symbol showing ------------------ */
 
 /**
  * Show symbols from given list with paging.
@@ -760,19 +834,7 @@ static void Symbols_Show(symbol_list_t* list, const char *sorttype)
 		(list == CpuSymbolsList ? "CPU" : "DSP"), sorttype);
 
 	for (entry = entries, i = 0; i < list->count; i++, entry++) {
-		switch (entry->type) {
-		case SYMTYPE_TEXT:
-			symchar = 'T';
-			break;
-		case SYMTYPE_DATA:
-			symchar = 'D';
-			break;
-		case SYMTYPE_BSS:
-			symchar = 'B';
-			break;
-		default:
-			symchar = '?';
-		}
+		symchar = symbol_char(entry->type);
 		fprintf(stderr, "0x%08x %c %s\n",
 			entry->address, symchar, entry->name);
 		if (i && i % 20 == 0) {
@@ -784,6 +846,69 @@ static void Symbols_Show(symbol_list_t* list, const char *sorttype)
 	}
 }
 
+/* ---------------- binary load handling ------------------ */
+
+
+/**
+ * Remove last opened program path.
+ */
+void Symbols_RemoveCurrentProgram(void)
+{
+	if (CurrentProgramPath) {
+		free(CurrentProgramPath);
+		CurrentProgramPath = NULL;
+
+		if (SymbolsAreForProgram) {
+			Symbols_Free(CpuSymbolsList);
+			CpuSymbolsList = NULL;
+		}
+	}
+	AutoLoadFailed = false;
+}
+
+/**
+ * Set last opened program path.
+ */
+void Symbols_ChangeCurrentProgram(FILE *fp, const char *path)
+{
+	if (is_atari_program(fp)) {
+		Symbols_RemoveCurrentProgram();
+		CurrentProgramPath = strdup(path);
+	}
+}
+
+/**
+ * Load symbols for last opened program.
+ */
+void Symbols_LoadCurrentProgram(void)
+{
+	/* symbols already loaded, program path missing or previous load failed? */
+	if (CpuSymbolsList || !CurrentProgramPath || AutoLoadFailed) {
+		return;
+	}
+	CpuSymbolsList = Symbols_Load(CurrentProgramPath, NULL, 0);
+	if (!CpuSymbolsList) {
+		AutoLoadFailed = true;
+	} else {
+		AutoLoadFailed = false;
+	}
+}
+
+/* ---------------- command parsing ------------------ */
+
+/**
+ * Readline match callback to list symbols subcommands.
+ * STATE = 0 -> different text from previous one.
+ * Return next match or NULL if no matches.
+ */
+char *Symbols_MatchCommand(const char *text, int state)
+{
+	static const char* subs[] = {
+		"addr", "free", "name", "prg"
+	};
+	return DebugUI_MatchHelper(subs, ARRAYSIZE(subs), text, state);
+}
+
 const char Symbols_Description[] =
 	"<filename|prg|addr|name|free> [<T offset> [<D offset> <B offset>]]\n"
 	"\tLoads symbol names and their addresses from the given file.\n"
@@ -823,8 +948,7 @@ int Symbols_Command(int nArgc, char *psArgs[])
 		maxaddr = 0;
 	}
 	if (nArgc < 2 || listtype == TYPE_NONE) {
-		DebugUI_PrintCmdHelp(psArgs[0]);
-		return DEBUGGER_CMDDONE;
+		return DebugUI_PrintCmdHelp(psArgs[0]);
 	}
 	file = psArgs[1];
 
@@ -858,7 +982,7 @@ int Symbols_Command(int nArgc, char *psArgs[])
 	}
 
 	if (strcmp(file, "prg") == 0) {
-		file = GemDOS_GetLastProgramPath();
+		file = CurrentProgramPath;
 		if (!file) {
 			fprintf(stderr, "ERROR: no program loaded (through GEMDOS HD emu)!\n");
 			return DEBUGGER_CMDDONE;
