@@ -80,4 +80,7 @@ void Crossbar_DmaRecordInHandShakeMode_Frame(Uint32 frame);
 /* Called by microphone.c */
 void Crossbar_GetMicrophoneDatas(Sint16 *micro_bufferL, Sint16 *micro_bufferR, Uint32 microBuffer_size);
 
+/* called by debugInfo.c */
+extern void Crossbar_Info(Uint32 dummy);
+
 #endif /* HATARI_CROSSBAR_H */
