@@ -26,6 +26,7 @@
 #include <string.h>
 #include <inttypes.h>
 #include <stdbool.h>
+#include <stdio.h>
 
 #include "dsp_core.h"
 #include "dsp_cpu.h"
@@ -677,6 +678,9 @@ static void opcode8h_0(void)
 		case 0x00008c:
 			dsp_enddo();
 			break;
+		default:
+			dsp_undefined();
+			break;
 	}
 }
 
