@@ -28,7 +28,7 @@ const char File_fileid[] = "Hatari file.c : " __DATE__ " " __TIME__;
 #include "str.h"
 #include "zip.h"
 
-#if defined(WIN32)
+#ifndef HAVE_FTELLO
 #define ftello ftell
 #endif
 
@@ -72,7 +72,7 @@ void File_AddSlashToEndFileName(char *pszFileName)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Does filename extension match? If so, return TRUE
+ * Does filename's extension match? If so, return TRUE
  */
 bool File_DoesFileExtensionMatch(const char *pszFileName, const char *pszExtension)
 {
@@ -87,6 +87,31 @@ bool File_DoesFileExtensionMatch(const char *pszFileName, const char *pszExtensi
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * If filename's extension matches, replace it with a new extension and
+ * copy the result in the new filename.
+ * Return TRUE if OK
+ */
+bool File_ChangeFileExtension(const char *Filename_old, const char *Extension_old , char *Filename_new , const char *Extension_new)
+{
+	if ( strlen ( Filename_old ) >= FILENAME_MAX - strlen ( Extension_new ) )
+		return false;					/* file name is already too long */
+
+	if ( strlen ( Filename_old ) < strlen ( Extension_old ) )
+		return false;
+
+	if ( !strcasecmp ( Filename_old + strlen(Filename_old) - strlen(Extension_old) , Extension_old ) )
+	{
+		strcpy ( Filename_new , Filename_old );
+		strcpy ( Filename_new + strlen ( Filename_new ) - strlen ( Extension_old ) , Extension_new );
+		return true;
+	}
+
+	return false;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Check if filename is from root
@@ -172,6 +197,7 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 	if (!filepath)
 		filepath = strdup(pszFileName);
 
+#if HAVE_LIBZ
 	/* Is it a gzipped file? */
 	if (File_DoesFileExtensionMatch(filepath, ".gz"))
 	{
@@ -188,6 +214,7 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 				if (gzread(hGzFile, tmp, sizeof(tmp)) < 0)
 				{
 					fprintf(stderr, "Failed to read gzip file!\n");
+					free(filepath);
 					return NULL;
 				}
 			}
@@ -208,6 +235,7 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 		pFile = ZIP_ReadFirstFile(filepath, &FileSize, ppszExts);
 	}
 	else          /* It is a normal file */
+#endif  /* HAVE_LIBZ */
 	{
 		FILE *hDiskFile;
 		/* Open and read normal file */
@@ -252,6 +280,7 @@ bool File_Save(const char *pszFileName, const Uint8 *pAddress, size_t Size, bool
 			return false;
 	}
 
+#if HAVE_LIBZ
 	/* Normal file or gzipped file? */
 	if (File_DoesFileExtensionMatch(pszFileName, ".gz"))
 	{
@@ -268,6 +297,7 @@ bool File_Save(const char *pszFileName, const Uint8 *pAddress, size_t Size, bool
 		}
 	}
 	else
+#endif  /* HAVE_LIBZ */
 	{
 		FILE *hDiskFile;
 		/* Create a normal file: */
