@@ -56,7 +56,7 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 #include "str.h"
 #include "hatari-glue.h"
 #include "maccess.h"
-
+#include "symbols.h"
 
 /* Maximum supported length of a GEMDOS path: */
 #define MAX_GEMDOS_PATH 256
@@ -138,7 +138,6 @@ static Uint16 nAttrSFirst;  /* File attribute for SFirst/Snext */
 
 /* last program opened by GEMDOS emulation */
 static bool PexecCalled;
-static char *LastProgramPath;
 
 #if defined(WIN32) && !defined(mkdir)
 #define mkdir(name,mode) mkdir(name)
@@ -368,7 +367,7 @@ static bool fsfirst_match(const char *pat, const char *name)
 			}
 			else
 			{
-				if (toupper(*p++) != toupper(*n++))
+				if (toupper((unsigned char)*p++) != toupper((unsigned char)*n++))
 					return false;
 			}
 		}
@@ -443,53 +442,21 @@ static void GemDOS_UnforceFileHandle(int i)
 }
 
 /*-----------------------------------------------------------------------*/
-/**
- * Return last host path used to execute a program
- * (used by debugger)
- */
-const char *GemDOS_GetLastProgramPath(void)
-{
-	return LastProgramPath;
-}
-
-static void GemDOS_RemoveLastProgram(void)
-{
-	if (LastProgramPath)
-	{
-		free(LastProgramPath);
-		LastProgramPath = NULL;
-	}
-}
 
 /**
  * If program was executed, store path to it
  * (should be called only by Fopen)
  */
-static void GemDOS_UpdateLastProgram(int Handle)
+static void GemDOS_UpdateCurrentProgram(int Handle)
 {
-	Uint16 magic = 0;
-	size_t items;
-	long oldpos;
-	FILE *fp;
-
 	/* only first Fopen after Pexec needs to be handled */
 	if (!PexecCalled)
 		return;
 	PexecCalled = false;
 
-	/* is file a TOS program? */
-	fp = FileHandles[Handle].FileHandle;
-	oldpos = ftell(fp);
-	fseek(fp, 0, SEEK_SET);
-	items = fread(&magic, sizeof(magic), 1, fp);
-	fseek(fp, oldpos, SEEK_SET);
-	if (items != 1 || SDL_SwapBE16(magic) != 0x601A)
-		return;
-
 	/* store program path */
-	if (LastProgramPath)
-		free(LastProgramPath);
-	LastProgramPath = strdup(FileHandles[Handle].szActualName);
+	Symbols_ChangeCurrentProgram(FileHandles[Handle].FileHandle,
+				     FileHandles[Handle].szActualName);
 }
 
 /*-----------------------------------------------------------------------*/
@@ -544,7 +511,7 @@ void GemDOS_Reset(void)
 	/* Reset */
 	bInitGemDOS = false;
 	CurrentDrive = nBootDrive;
-	GemDOS_RemoveLastProgram();
+	Symbols_RemoveCurrentProgram();
 	pDTA = NULL;
 }
 
@@ -560,7 +527,7 @@ static bool GEMDOS_DoesHostDriveFolderExist(char* lpstrPath, int iDrive)
 	{
 		/* Try lower case drive letter instead */
 		int	iIndex = strlen(lpstrPath)-1;
-		lpstrPath[iIndex] = tolower(lpstrPath[iIndex]);
+		lpstrPath[iIndex] = tolower((unsigned char)lpstrPath[iIndex]);
 	}
 
 	/* Check the file/folder is accessible (security basis) */
@@ -616,7 +583,7 @@ static bool GemDOS_DetermineMaxPartitions(int *pnMaxDrives)
 		last = 0;
 		for (i = 0; i < count; i++)
 		{
-			letter = toupper(files[i]->d_name[0]);
+			letter = toupper((unsigned char)files[i]->d_name[0]);
 			if (!letter || letter == '.')
 			{
 				/* Ignore hidden files like "." and ".." */
@@ -948,7 +915,7 @@ static int GemDOS_FindDriveNumber(char *pszFileName)
 	/* Does have 'A:' or 'C:' etc.. at start of string? */
 	if (pszFileName[0] != '\0' && pszFileName[1] == ':')
 	{
-		char letter = toupper(pszFileName[0]);
+		char letter = toupper((unsigned char)pszFileName[0]);
 		if (letter >= 'A' && letter <= 'Z')
 			return (letter-'A');
 	}
@@ -1061,7 +1028,7 @@ static int clip_to_83(char *name)
 	int diff, len;
 	char *dot;
 	
-	dot = strrchr(name, '.');
+	dot = strchr(name, '.');
 	if (dot) {
 		diff = strlen(dot) - 4;
 		if (diff > 0)
@@ -1417,7 +1384,8 @@ static bool GemDOS_SetDrv(Uint32 Params)
 	/* Read details from stack for our own use */
 	CurrentDrive = STMemory_ReadWord(Params);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x0E Dsetdrv(0x%x)\n", (int)CurrentDrive);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x0E Dsetdrv(0x%x) at PC=0x%X\n", (int)CurrentDrive,
+		  M68000_GetPC());
 
 	/* Still re-direct to TOS */
 	return false;
@@ -1434,7 +1402,8 @@ static bool GemDOS_SetDTA(Uint32 Params)
 	/*  Look up on stack to find where DTA is */
 	Uint32 nDTA = STMemory_ReadLong(Params);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x1A Fsetdta(0x%x)\n", nDTA);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x1A Fsetdta(0x%x) at PC 0x%X\n", nDTA,
+		  M68000_GetPC());
 
 	if (STMemory_ValidArea(nDTA, sizeof(DTA)))
 	{
@@ -1467,7 +1436,8 @@ static bool GemDOS_DFree(Uint32 Params)
 	Drive = STMemory_ReadWord(Params+SIZE_LONG);
 
 	/* Note: Drive = 0 means current drive, 1 = A:, 2 = B:, 3 = C:, etc. */
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x36 Dfree(0x%x, %i)\n", Address, Drive);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x36 Dfree(0x%x, %i) at PC 0x%X\n", Address, Drive,
+		  M68000_GetPC());
 	if (Drive == 0)
 		Drive = CurrentDrive;
 	else
@@ -1498,7 +1468,7 @@ static bool GemDOS_DFree(Uint32 Params)
 		Free = Free/1024 * buf.f_bsize;
 
 		/* TOS version limits based on:
-		 *   http://www.seimet.de/atari/en/hddriverfaq.html
+		 *   http://hddriver.seimet.de/en/faq.html
 		 */
 		if (TosVersion >= 0x0400)
 		{
@@ -1550,7 +1520,8 @@ static bool GemDOS_MkDir(Uint32 Params)
 	/* Find directory to make */
 	pDirName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x39 Dcreate(\"%s\")\n", pDirName);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x39 Dcreate(\"%s\") at PC 0x%X\n", pDirName,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pDirName);
 
@@ -1602,7 +1573,8 @@ static bool GemDOS_RmDir(Uint32 Params)
 	/* Find directory to make */
 	pDirName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3A Ddelete(\"%s\")\n", pDirName);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3A Ddelete(\"%s\") at PC 0x%X\n", pDirName,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pDirName);
 
@@ -1656,7 +1628,8 @@ static bool GemDOS_ChDir(Uint32 Params)
 	/* Find new directory */
 	pDirName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3B Dsetpath(\"%s\")\n", pDirName);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3B Dsetpath(\"%s\") at PC 0x%X\n", pDirName,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pDirName);
 
@@ -1745,7 +1718,8 @@ static bool GemDOS_Create(Uint32 Params)
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 	Mode = STMemory_ReadWord(Params+SIZE_LONG);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3C Fcreate(\"%s\", 0x%x)\n", pszFileName, Mode);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3C Fcreate(\"%s\", 0x%x) at PC 0x%X\n", pszFileName, Mode,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pszFileName);
 
@@ -1868,7 +1842,8 @@ static bool GemDOS_Open(Uint32 Params)
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 	Mode = STMemory_ReadWord(Params+SIZE_LONG);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3D Fopen(\"%s\", %s)\n", pszFileName, Modes[Mode&0x03].desc);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3D Fopen(\"%s\", %s) at PC=0x%X\n", pszFileName, Modes[Mode&0x03].desc,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pszFileName);
 
@@ -1929,7 +1904,7 @@ static bool GemDOS_Open(Uint32 Params)
 			 sizeof(FileHandles[Index].szActualName),
 			 "%s", szActualFileName);
 
-		GemDOS_UpdateLastProgram(Index);
+		GemDOS_UpdateCurrentProgram(Index);
 
 		/* Return valid ST file handle from our range (BASE_FILEHANDLE upwards) */
 		Regs[REG_D0] = Index+BASE_FILEHANDLE;
@@ -1969,7 +1944,8 @@ static bool GemDOS_Close(Uint32 Params)
 	/* Find our handle - may belong to TOS */
 	Handle = STMemory_ReadWord(Params);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3E Fclose(%i)\n", Handle);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3E Fclose(%i) at PC 0x%X\n", Handle,
+		  M68000_GetPC());
 
 	/* Get internal handle */
 	if ((Handle = GemDOS_GetValidFileHandle(Handle)) < 0)
@@ -2015,8 +1991,9 @@ static bool GemDOS_Read(Uint32 Params)
 	Addr = STMemory_ReadLong(Params+SIZE_WORD+SIZE_LONG);
 	pBuffer = (char *)STRAM_ADDR(Addr);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3F Fread(%i, %i, 0x%x)\n", 
-	          Handle, Size, Addr);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x3F Fread(%i, %i, 0x%x) at PC 0x%X\n",
+	          Handle, Size, Addr,
+		  M68000_GetPC());
 
 	/* Get internal handle */
 	if ((Handle = GemDOS_GetValidFileHandle(Handle)) < 0)
@@ -2089,8 +2066,9 @@ static bool GemDOS_Write(Uint32 Params)
 	Addr = STMemory_ReadLong(Params+SIZE_WORD+SIZE_LONG);
 	pBuffer = (char *)STRAM_ADDR(Addr);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x40 Fwrite(%i, %i, 0x%x)\n", 
-	          Handle, Size, Addr);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x40 Fwrite(%i, %i, 0x%x) at PC 0x%X\n",
+	          Handle, Size, Addr,
+		  M68000_GetPC());
 
 	/* Get internal handle */
 	if ((Handle = GemDOS_GetValidFileHandle(Handle)) < 0)
@@ -2145,7 +2123,8 @@ static bool GemDOS_FDelete(Uint32 Params)
 	/* Find filename */
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x41 Fdelete(\"%s\")\n", pszFileName);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x41 Fdelete(\"%s\") at PC 0x%X\n", pszFileName,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pszFileName);
 
@@ -2203,7 +2182,8 @@ static bool GemDOS_LSeek(Uint32 Params)
 	Handle = STMemory_ReadWord(Params+SIZE_LONG);
 	Mode = STMemory_ReadWord(Params+SIZE_LONG+SIZE_WORD);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x42 Fseek(%li, %i, %i)\n", Offset, Handle, Mode);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x42 Fseek(%li, %i, %i) at PC 0x%X\n", Offset, Handle, Mode,
+		  M68000_GetPC());
 
 	/* get internal handle */
 	if ((Handle = GemDOS_GetValidFileHandle(Handle)) < 0)
@@ -2270,8 +2250,9 @@ static bool GemDOS_Fattrib(Uint32 Params)
 	nRwFlag = STMemory_ReadWord(Params+SIZE_LONG);
 	nAttrib = STMemory_ReadWord(Params+SIZE_LONG+SIZE_WORD);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x43 Fattrib(\"%s\", %d, 0x%x)\n",
-	          psFileName, nRwFlag, nAttrib);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x43 Fattrib(\"%s\", %d, 0x%x) at PC 0x%X\n",
+	          psFileName, nRwFlag, nAttrib,
+		  M68000_GetPC());
 
 	if (!ISHARDDRIVE(nDrive))
 	{
@@ -2370,7 +2351,8 @@ static bool GemDOS_Force(Uint32 Params)
 	std = STMemory_ReadWord(Params);
         own = STMemory_ReadWord(Params+SIZE_WORD);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x46 Fforce(%d, %d)\n", std, own);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x46 Fforce(%d, %d) at PC 0x%X\n", std, own,
+		  M68000_GetPC());
 
 	/* Get internal handle */
 	if (std > own)
@@ -2412,7 +2394,8 @@ static bool GemDOS_GetDir(Uint32 Params)
 	Drive = STMemory_ReadWord(Params+SIZE_LONG);
 
 	/* Note: Drive = 0 means current drive, 1 = A:, 2 = B:, 3 = C:, etc. */
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x47 Dgetpath(0x%x, %i)\n", Address, (int)Drive);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x47 Dgetpath(0x%x, %i) at PC 0x%X\n", Address, (int)Drive,
+		  M68000_GetPC());
 	if (Drive == 0)
 		Drive = CurrentDrive;
 	else
@@ -2480,11 +2463,13 @@ static int GemDOS_Pexec(Uint32 Params)
 			name = (const char *)STRAM_ADDR(fname);
 			cmd = (const char *)STRAM_ADDR(cmdline);
 			cmdlen = *cmd++;
-			fprintf(TraceFile, "GEMDOS 0x4B Pexec(%i, \"%s\", [%d]\"%s\", 0x%x)\n", Mode, name, cmdlen, cmdlen?cmd:"", env_string);
+			fprintf(TraceFile, "GEMDOS 0x4B Pexec(%i, \"%s\", [%d]\"%s\", 0x%x) at PC 0x%X\n", Mode, name, cmdlen, cmdlen?cmd:"", env_string,
+				M68000_GetPC());
 		}
 		else
 		{
-			fprintf(TraceFile, "GEMDOS 0x4B Pexec(%i, 0x%x, 0x%x, 0x%x)\n", Mode, fname, cmdline, env_string);
+			fprintf(TraceFile, "GEMDOS 0x4B Pexec(%i, 0x%x, 0x%x, 0x%x) at PC 0x%X\n", Mode, fname, cmdline, env_string,
+				M68000_GetPC());
 		}
 	}
 
@@ -2529,7 +2514,7 @@ static bool GemDOS_SNext(void)
 	int Index;
 	int ret;
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4F Fsnext()\n");
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4F Fsnext() at PC 0x%X\n" , M68000_GetPC());
 
 	/* Refresh pDTA pointer (from the current basepage) */
 	nDTA = STMemory_ReadLong(STMemory_ReadLong(act_pd)+32);
@@ -2606,7 +2591,8 @@ static bool GemDOS_SFirst(Uint32 Params)
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params));
 	nAttrSFirst = STMemory_ReadWord(Params+SIZE_LONG);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4E Fsfirst(\"%s\", 0x%x)\n", pszFileName, nAttrSFirst);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4E Fsfirst(\"%s\", 0x%x) at PC 0x%X\n", pszFileName, nAttrSFirst,
+		  M68000_GetPC());
 
 	Drive = GemDOS_FileName2HardDriveID(pszFileName);
 	if (!ISHARDDRIVE(Drive))
@@ -2729,7 +2715,8 @@ static bool GemDOS_Rename(Uint32 Params)
 	pszOldFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
 	pszNewFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD+SIZE_LONG));
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x56 Frename(\"%s\", \"%s\")\n", pszOldFileName, pszNewFileName);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x56 Frename(\"%s\", \"%s\") at PC 0x%X\n", pszOldFileName, pszNewFileName,
+		  M68000_GetPC());
 
 	NewDrive = GemDOS_FileName2HardDriveID(pszNewFileName);
 	OldDrive = GemDOS_FileName2HardDriveID(pszOldFileName);
@@ -2778,8 +2765,9 @@ static bool GemDOS_GSDToF(Uint32 Params)
 	Handle = STMemory_ReadWord(Params+SIZE_LONG);
 	Flag = STMemory_ReadWord(Params+SIZE_LONG+SIZE_WORD);
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x57 Fdatime(0x%x, %i, %i)\n", pBuffer,
-	          Handle, Flag);
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x57 Fdatime(0x%x, %i, %i) at PC 0x%X\n", pBuffer,
+	          Handle, Flag,
+		  M68000_GetPC());
 
 	/* get internal handle */
 	if ((Handle = GemDOS_GetValidFileHandle(Handle)) < 0)
@@ -2868,9 +2856,10 @@ static void GemDOS_TerminateClose(void)
  */
 static bool GemDOS_Pterm0(Uint32 Params)
 {
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x00 Pterm0()\n");
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x00 Pterm0() at PC 0x%X\n",
+		  M68000_GetPC());
 	GemDOS_TerminateClose();
-	GemDOS_RemoveLastProgram();
+	Symbols_RemoveCurrentProgram();
 	return false;
 }
 
@@ -2880,8 +2869,9 @@ static bool GemDOS_Pterm0(Uint32 Params)
  */
 static bool GemDOS_Ptermres(Uint32 Params)
 {
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x31 Ptermres(0x%X, %d)\n",
-		  STMemory_ReadLong(Params), STMemory_ReadWord(Params+SIZE_WORD));
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x31 Ptermres(0x%X, %hd) at PC 0x%X\n",
+		  STMemory_ReadLong(Params), (Sint16)STMemory_ReadWord(Params+SIZE_WORD),
+		  M68000_GetPC());
 	GemDOS_TerminateClose();
 	return false;
 }
@@ -2892,10 +2882,11 @@ static bool GemDOS_Ptermres(Uint32 Params)
  */
 static bool GemDOS_Pterm(Uint32 Params)
 {
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4C Pterm(%d)\n",
-		  STMemory_ReadWord(Params));
+	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4C Pterm(%hd) at PC 0x%X\n",
+		  (Sint16)STMemory_ReadWord(Params),
+		  M68000_GetPC());
 	GemDOS_TerminateClose();
-	GemDOS_RemoveLastProgram();
+	Symbols_RemoveCurrentProgram();
 	return false;
 }
 
@@ -3236,8 +3227,9 @@ void GemDOS_OpCode(void)
 	case 0x2A:	/* Tgetdate */
 	case 0x2C:	/* Tgettime */
 		/* commands with no args */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX %s()\n",
-			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall));
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX %s() at PC 0x%X\n",
+			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall),
+			  M68000_GetPC());
 		break;
 		
 	case 0x02:	/* Cconout */
@@ -3248,9 +3240,10 @@ void GemDOS_OpCode(void)
 	case 0x2d:	/* Tsettime */
 	case 0x45:	/* Fdup */
 		/* commands taking single word */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX %s(0x%hX)\n",
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX %s(0x%hX) at PC 0x%X\n",
 			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall),
-			  STMemory_ReadWord(Params));
+			  STMemory_ReadWord(Params),
+			  M68000_GetPC());
 		break;
 
 	case 0x09:	/* Cconws */
@@ -3259,35 +3252,40 @@ void GemDOS_OpCode(void)
 	case 0x48:	/* Malloc */
 	case 0x49:	/* Mfree */
 		/* commands taking long/pointer */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX %s(0x%X)\n",
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX %s(0x%X) at PC 0x%X\n",
 			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall),
-			  STMemory_ReadLong(Params));
+			  STMemory_ReadLong(Params),
+			  M68000_GetPC());
 		break;
 
 	case 0x44:	/* Mxalloc */
 		/* commands taking long/pointer + word */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x44 Mxalloc(0x%X, 0x%hX)\n",
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x44 Mxalloc(0x%X, 0x%hX) at PC 0x%X\n",
 			  STMemory_ReadLong(Params),
-			  STMemory_ReadWord(Params+SIZE_LONG));
+			  STMemory_ReadWord(Params+SIZE_LONG),
+			  M68000_GetPC());
 		break;
 	case 0x14:	/* Maddalt */
 		/* commands taking 2 longs/pointers */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x14 Maddalt(0x%X, 0x%X)\n",
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x14 Maddalt(0x%X, 0x%X) at PC 0x%X\n",
 			  STMemory_ReadLong(Params),
-			  STMemory_ReadLong(Params+SIZE_LONG));
+			  STMemory_ReadLong(Params+SIZE_LONG),
+			  M68000_GetPC());
 	case 0x4A:	/* Mshrink */
 		/* Mshrink's two pointers are prefixed by reserved zero word:
 		 * http://toshyp.atari.org/en/00500c.html#Bindings_20for_20Mshrink
 		 */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4A Mshrink(0x%X, 0x%X)\n",
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4A Mshrink(0x%X, 0x%X) at PC 0x%X\n",
 			  STMemory_ReadLong(Params+SIZE_WORD),
-			  STMemory_ReadLong(Params+SIZE_WORD+SIZE_LONG));
+			  STMemory_ReadLong(Params+SIZE_WORD+SIZE_LONG),
+			  M68000_GetPC());
 		break;
 
 	default:
 		/* rest of commands */
-		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX (%s)\n",
-			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall));
+		LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x%2hX (%s) at PC 0x%X\n",
+			  GemDOSCall, GemDOS_Opcode2Name(GemDOSCall),
+			  M68000_GetPC());
 	}
 
 	switch(Finished)
@@ -3337,10 +3335,12 @@ void GemDOS_Boot(void)
 {
 	bInitGemDOS = true;
 
-	LOG_TRACE(TRACE_OS_GEMDOS, "Gemdos_Boot()\n" );
+	LOG_TRACE(TRACE_OS_GEMDOS, "Gemdos_Boot() at PC 0x%X\n", M68000_GetPC() );
 
-	/* install our gemdos handler, if -e or --harddrive option used */
-	if (!GEMDOS_EMU_ON)
+	/* install our gemdos handler, if -e or --harddrive option used,
+	 * or user wants to do GEMDOS tracing
+	 */
+	if (!GEMDOS_EMU_ON && !(LogTraceFlags & TRACE_OS_GEMDOS))
 		return;
 
 	/* Get the address of the p_run variable that points to the actual basepage */
