@@ -13,6 +13,7 @@ typedef struct
 {
   char sLogFileName[FILENAME_MAX];
   char sTraceFileName[FILENAME_MAX];
+  int nExceptionDebugMask;
   int nTextLogLevel;
   int nAlertDlgLogLevel;
   bool bConfirmQuit;
@@ -159,6 +160,10 @@ typedef struct
 {
   bool bAutoInsertDiskB;
   bool FastFloppy;			/* true to speed up FDC emulation */
+  bool EnableDriveA;
+  bool EnableDriveB;
+  int  DriveA_NumberOfHeads;
+  int  DriveB_NumberOfHeads;
   WRITEPROTECTION nWriteProtection;
   char szDiskZipPath[MAX_FLOPPYDRIVES][FILENAME_MAX];
   char szDiskFileName[MAX_FLOPPYDRIVES][FILENAME_MAX];
@@ -181,18 +186,25 @@ typedef struct
 {
   int nHardDiskDir;
   bool bUseHardDiskDirectories;
-  bool bUseHardDiskImage;
   bool bUseIdeMasterHardDiskImage;
   bool bUseIdeSlaveHardDiskImage;
   WRITEPROTECTION nWriteProtection;
   GEMDOS_CHR_CONV nGemdosCase;
   bool bBootFromHardDisk;
   char szHardDiskDirectories[MAX_HARDDRIVES][FILENAME_MAX];
-  char szHardDiskImage[FILENAME_MAX];
   char szIdeMasterHardDiskImage[FILENAME_MAX];
   char szIdeSlaveHardDiskImage[FILENAME_MAX];
 } CNF_HARDDISK;
 
+/* SCSI/ACSI configuration */
+#define MAX_ACSI_DEVS 8
+
+typedef struct
+{
+  bool bUseDevice;
+  char sDeviceFile[FILENAME_MAX];
+} CNF_SCSIDEV;
+
 /* Falcon register $FFFF8006 bits 6 & 7 (mirrored in $FFFF82C0 bits 0 & 1):
  * 00 Monochrome
  * 01 RGB - Colormonitor
@@ -325,6 +337,7 @@ typedef struct
   CNF_MEMORY Memory;
   CNF_DISKIMAGE DiskImage;
   CNF_HARDDISK HardDisk;
+  CNF_SCSIDEV Acsi[MAX_ACSI_DEVS];
   CNF_ROM Rom;
   CNF_RS232 RS232;
   CNF_PRINTER Printer;
