@@ -1,26 +1,13 @@
 /*
   Hatari - joy.h
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 */
 
 #ifndef HATARI_JOY_H
 #define HATARI_JOY_H
 
-typedef struct
-{
-	int XPos,YPos;                /* the actually read axis values in range of -32768...0...32767 */
-	int XAxisID,YAxisID;          /* the IDs of the physical PC joystick's axis to be used to gain ST joystick axis input */
-	int Buttons;                  /* JOY_BUTTON1 */
-} JOYREADING;
-
-typedef struct
-{
-    char SDLJoystickName[50];
-    int XAxisID,YAxisID;           /* the IDs associated with a certain SDL joystick */
-} JOYAXISMAPPING;
-
 enum
 {
 	JOYSTICK_SPACE_NULL,          /* Not up/down */
@@ -35,7 +22,7 @@ enum
 	JOYID_STEPADA,
 	JOYID_STEPADB,
 	JOYID_PARPORT1,
-	JOYID_PARPORT2,
+	JOYID_PARPORT2
 };
 
 #define JOYRANGE_UP_VALUE     -16384     /* Joystick ranges in XY */
