@@ -35,7 +35,9 @@
 
 #define CALL_VAR(func)  { ((void(*)(void))func)(); }
 
+#ifndef ARRAYSIZE
 #define ARRAYSIZE(x) (int)(sizeof(x)/sizeof(x[0]))
+#endif
 
 /* 68000 operand sizes */
 #define SIZE_BYTE  1
@@ -49,8 +51,9 @@ extern bool bQuitProgram;
 
 extern bool Main_PauseEmulation(bool visualize);
 extern bool Main_UnPauseEmulation(void);
-extern void Main_RequestQuit(void);
+extern void Main_RequestQuit(int exitval);
 extern void Main_SetRunVBLs(Uint32 vbls);
+extern bool Main_SetVBLSlowdown(int factor);
 extern void Main_WaitOnVbl(void);
 extern void Main_WarpMouse(int x, int y);
 extern void Main_EventHandler(void);
