@@ -1,8 +1,8 @@
 /*
   Hatari - screen.h
 
-  This file is distributed under the GNU Public License, version 2 or at your
-  option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 */
 
 #ifndef HATARI_SCREEN_H
@@ -104,4 +104,6 @@ extern void Screen_ReturnFromFullScreen(void);
 extern void Screen_ModeChanged(void);
 extern bool Screen_Draw(void);
 
+extern bool bTTSampleHold;      /* TT special video mode */
+
 #endif  /* ifndef HATARI_SCREEN_H */
