@@ -197,6 +197,22 @@ extern void Video_Color12_WriteWord(void);
 extern void Video_Color13_WriteWord(void);
 extern void Video_Color14_WriteWord(void);
 extern void Video_Color15_WriteWord(void);
+extern void Video_Color0_ReadWord(void);
+extern void Video_Color1_ReadWord(void);
+extern void Video_Color2_ReadWord(void);
+extern void Video_Color3_ReadWord(void);
+extern void Video_Color4_ReadWord(void);
+extern void Video_Color5_ReadWord(void);
+extern void Video_Color6_ReadWord(void);
+extern void Video_Color7_ReadWord(void);
+extern void Video_Color8_ReadWord(void);
+extern void Video_Color9_ReadWord(void);
+extern void Video_Color10_ReadWord(void);
+extern void Video_Color11_ReadWord(void);
+extern void Video_Color12_ReadWord(void);
+extern void Video_Color13_ReadWord(void);
+extern void Video_Color14_ReadWord(void);
+extern void Video_Color15_ReadWord(void);
 extern void Video_ShifterMode_WriteByte(void);
 extern void Video_HorScroll_Write_8264(void);
 extern void Video_HorScroll_Write_8265(void);
@@ -205,4 +221,6 @@ extern void Video_TTShiftMode_WriteWord(void);
 extern void Video_TTColorRegs_WriteWord(void);
 extern void Video_TTColorSTRegs_WriteWord(void);
 
+extern void Video_Info(Uint32 dummy);
+
 #endif  /* HATARI_VIDEO_H */
