@@ -19,6 +19,7 @@ const char Joy_fileid[] = "Hatari joy.c : " __DATE__ " " __TIME__;
 #include "log.h"
 #include "screen.h"
 #include "video.h"
+#include "statusbar.h"
 
 #define JOY_BUTTON1  1
 #define JOY_BUTTON2  2
@@ -76,8 +77,9 @@ void Joy_Init(void)
 	/* FIXME: Read those settings from a configuration file and make them tunable from the GUI. */
 	static const JOYAXISMAPPING AxisMappingTable [] =
 	{
-       		/* USB game pad with ID ID 0079:0011, sold by Speedlink */
-		{"USB Gamepad" , 3, 4},
+		/* Example entry for mapping joystick axis for a certain device: */
+		/* USB game pad with ID ID 0079:0011, sold by Speedlink with axis 3 and 4 used */
+		/*{"USB Gamepad" , 3, 4}, */
 		/* Default entry used if no other SDL joystick name does match (should be last of this list) */
 		{"*DEFAULT*" , 0, 1},
 	};
@@ -345,6 +347,7 @@ void Joy_ToggleCursorEmulation(void)
 	default:  /* neither in port 0 or 1, enable cursor emu to port 1 */
 		ConfigureParams.Joysticks.Joy[1].nJoystickMode = JOYSTICK_KEYBOARD;
 	}
+	Statusbar_UpdateInfo();
 }
 
 
