@@ -545,7 +545,7 @@ static char Keymap_GetKeyPadScanCode(SDL_keysym* pKeySym)
 /**
  * Remap SDL Key to ST Scan code
  */
-char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
+static char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
 {
 	if (pKeySym->sym >= SDLK_LAST)  return -1; /* Avoid illegal keys */
 
@@ -680,9 +680,7 @@ static bool Keymap_DebounceSTKey(char STScanCode)
  */
 void Keymap_DebounceAllKeys(void)
 {
-	SDLKey key;
-	char STScanCode;
-	SDL_keysym tmpKeySym;
+	uint8_t nScanCode;
 
 	/* Return if we aren't in fast forward or have not disabled key repeat */
 	if ((ConfigureParams.System.bFastForward == false)
@@ -691,24 +689,17 @@ void Keymap_DebounceAllKeys(void)
 		return;
 	}
 
-	tmpKeySym.mod = KMOD_NONE;
-
-	/* Now run through each PC key looking for ones held down */
-	for (key = SDLK_FIRST; key < SDLK_LAST; key++)
+	/* Now run through each key looking for ones held down */
+	for (nScanCode = 1; nScanCode <= KBD_MAX_SCANCODE; nScanCode++)
 	{
 		/* Is key held? */
-		if (Keyboard.KeyStates[key])
+		if (Keyboard.KeyStates[nScanCode])
 		{
-			tmpKeySym.sym = key;
-			tmpKeySym.scancode = 0;
-
-			/* Get scan code */
-			STScanCode = Keymap_RemapKeyToSTScanCode(&tmpKeySym);
-			if (STScanCode != (char)-1)
+			/* Does this require de-bouncing? */
+			if (Keymap_DebounceSTKey(nScanCode))
 			{
-				/* Does this require de-bouncing? */
-				if (Keymap_DebounceSTKey(STScanCode))
-					Keymap_KeyUp(&tmpKeySym);
+				IKBD_PressSTKey(nScanCode, false);
+				Keyboard.KeyStates[nScanCode] = false;
 			}
 		}
 	}
@@ -722,8 +713,7 @@ void Keymap_DebounceAllKeys(void)
  */
 void Keymap_KeyDown(SDL_keysym *sdlkey)
 {
-	bool bPreviousKeyState;
-	char STScanCode;
+	uint8_t STScanCode;
 	int symkey = sdlkey->sym;
 	int modkey = sdlkey->mod;
 
@@ -745,15 +735,15 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
 		return;
 	}
 
-	/* Set down */
-	bPreviousKeyState = Keyboard.KeyStates[symkey];
-	Keyboard.KeyStates[symkey] = true;
-
 	STScanCode = Keymap_RemapKeyToSTScanCode(sdlkey);
-	if (STScanCode != (char)-1)
+	if (STScanCode != (uint8_t)-1)
 	{
-		if (!bPreviousKeyState)
+		if (!Keyboard.KeyStates[STScanCode])
+		{
+			/* Set down */
+			Keyboard.KeyStates[STScanCode] = true;
 			IKBD_PressSTKey(STScanCode, true);
+		}
 	}
 }
 
@@ -764,7 +754,7 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
  */
 void Keymap_KeyUp(SDL_keysym *sdlkey)
 {
-	char STScanCode;
+	uint8_t STScanCode;
 	int symkey = sdlkey->sym;
 	int modkey = sdlkey->mod;
 
@@ -788,17 +778,16 @@ void Keymap_KeyUp(SDL_keysym *sdlkey)
 		return;
 	}
 
+	STScanCode = Keymap_RemapKeyToSTScanCode(sdlkey);
 	/* Release key (only if was pressed) */
-	if (Keyboard.KeyStates[symkey])
+	if (STScanCode != (uint8_t)-1)
 	{
-		STScanCode = Keymap_RemapKeyToSTScanCode(sdlkey);
-		if (STScanCode != (char)-1)
+		if (Keyboard.KeyStates[STScanCode])
 		{
 			IKBD_PressSTKey(STScanCode, false);
+			Keyboard.KeyStates[STScanCode] = false;
 		}
 	}
-
-	Keyboard.KeyStates[symkey] = false;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -811,12 +800,12 @@ void Keymap_SimulateCharacter(char asckey, bool press)
 
 	sdlkey.mod = KMOD_NONE;
 	sdlkey.scancode = 0;
-	if (isupper(asckey)) {
+	if (isupper((unsigned char)asckey)) {
 		if (press) {
 			sdlkey.sym = SDLK_LSHIFT;
 			Keymap_KeyDown(&sdlkey);
 		}
-		sdlkey.sym = tolower(asckey);
+		sdlkey.sym = tolower((unsigned char)asckey);
 		sdlkey.mod = KMOD_LSHIFT;
 	} else {
 		sdlkey.sym = asckey;
@@ -825,7 +814,7 @@ void Keymap_SimulateCharacter(char asckey, bool press)
 		Keymap_KeyDown(&sdlkey);
 	} else {
 		Keymap_KeyUp(&sdlkey);
-		if (isupper(asckey)) {
+		if (isupper((unsigned char)asckey)) {
 			sdlkey.sym = SDLK_LSHIFT;
 			Keymap_KeyUp(&sdlkey);
 		}
