@@ -21,7 +21,9 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "audio.h"
 #include "joy.h"
 #include "floppy.h"
+#include "floppy_ipf.h"
 #include "gemdos.h"
+#include "fdc.h"
 #include "hdc.h"
 #include "ide.h"
 #include "acia.h"
@@ -61,18 +63,22 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include <sys/time.h>
 #endif
 
+#ifdef WIN32
+#include "gui-win/opencon.h"
+#endif
 
 bool bQuitProgram = false;                /* Flag to quit program cleanly */
+static int nQuitValue;                    /* exit value */
 
 static Uint32 nRunVBLs;                   /* Whether and how many VBLS to run before exit */
 static Uint32 nFirstMilliTick;            /* Ticks when VBL counting started */
 static Uint32 nVBLCount;                  /* Frame count */
+static int nVBLSlowdown = 1;		  /* host VBL wait multiplier */
 
 static bool bEmulationActive = true;      /* Run emulation when started */
 static bool bAccurateDelays;              /* Host system has an accurate SDL_Delay()? */
 static bool bIgnoreNextMouseMotion = false;  /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
 
-
 /*-----------------------------------------------------------------------*/
 /**
  * Return current time as millisecond for performance measurements.
@@ -117,12 +123,12 @@ static Uint32 Main_GetTicks(void)
 
 static Sint64	Time_GetTicks ( void )
 {
-        Sint64		ticks_micro;
+	Sint64	ticks_micro;
 
 #if HAVE_GETTIMEOFDAY
-        struct timeval	now;
-        gettimeofday ( &now , NULL );
-        ticks_micro = (Sint64)now.tv_sec * 1000000 + now.tv_usec;
+	struct timeval	now;
+	gettimeofday ( &now , NULL );
+	ticks_micro = (Sint64)now.tv_sec * 1000000 + now.tv_usec;
 #else
 	ticks_micro = (Sint64)SDL_GetTicks() * 1000;		/* milli sec -> micro sec */
 #endif
@@ -189,7 +195,7 @@ bool Main_PauseEmulation(bool visualize)
 		
 		Statusbar_AddMessage("Emulation paused", 100);
 		/* make sure msg gets shown */
-		Statusbar_Update(sdlscrn);
+		Statusbar_Update(sdlscrn, true);
 
 		if (bGrabMouse && !bInFullScreen)
 			/* Un-grab mouse pointer in windowed mode */
@@ -226,7 +232,7 @@ bool Main_UnPauseEmulation(void)
 /**
  * Optionally ask user whether to quit and set bQuitProgram accordingly
  */
-void Main_RequestQuit(void)
+void Main_RequestQuit(int exitval)
 {
 	if (ConfigureParams.Memory.bAutoSave)
 	{
@@ -248,6 +254,7 @@ void Main_RequestQuit(void)
 		/* Assure that CPU core shuts down */
 		M68000_SetSpecial(SPCFLAG_BRK);
 	}
+	nQuitValue = exitval;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -262,6 +269,21 @@ void Main_SetRunVBLs(Uint32 vbls)
 	nVBLCount = 0;
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Set VBL wait slowdown factor/multiplayer
+ */
+bool Main_SetVBLSlowdown(int factor)
+{
+	if (factor < 1 || factor > 8) {
+		fprintf(stderr, "ERROR: invalid VBL slowdown factor %d, should be 1-8!\n", factor);
+		return false;
+	}
+	fprintf(stderr, "Slow down host VBL wait by factor of %d.\n", factor);
+	nVBLSlowdown = factor;
+	return true;
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function waits on each emulated VBL to synchronize the real time
@@ -289,18 +311,21 @@ void Main_WaitOnVbl(void)
 
 //	FrameDuration_micro = (Sint64) ( 1000000.0 / nScreenRefreshRate + 0.5 );	/* round to closest integer */
 	FrameDuration_micro = ClocksTimings_GetVBLDuration_micro ( ConfigureParams.System.nMachineType , nScreenRefreshRate );
+	FrameDuration_micro *= nVBLSlowdown;
 	CurrentTicks = Time_GetTicks();
 
-	if ( DestTicks == 0 )					/* first call, init DestTicks */
+	if (DestTicks == 0)			/* on first call, init DestTicks */
+	{
 		DestTicks = CurrentTicks + FrameDuration_micro;
+	}
 
-	DestTicks += pulse_swallowing_count; /* audio.c - Audio_CallBack() */
+	DestTicks += pulse_swallowing_count;	/* audio.c - Audio_CallBack() */
 
 	nDelay = DestTicks - CurrentTicks;
 
 	/* Do not wait if we are in fast forward mode or if we are totally out of sync */
 	if (ConfigureParams.System.bFastForward == true
-	        || nDelay < -4*FrameDuration_micro)
+	    || nDelay < -4*FrameDuration_micro || nDelay > 50*FrameDuration_micro)
 	{
 		if (ConfigureParams.System.bFastForward == true)
 		{
@@ -345,6 +370,10 @@ void Main_WaitOnVbl(void)
 	{
 		CurrentTicks = Time_GetTicks();
 		nDelay = DestTicks - CurrentTicks;
+		/* If the delay is still bigger than one frame, somebody
+		 * played tricks with the system clock and we have to abort */
+		if (nDelay > FrameDuration_micro)
+			break;
 	}
 
 //printf ( "tick %lld\n" , CurrentTicks );
@@ -477,7 +506,7 @@ void Main_EventHandler(void)
 		{
 
 		 case SDL_QUIT:
-			Main_RequestQuit();
+			Main_RequestQuit(0);
 			break;
 			
 		 case SDL_MOUSEMOTION:               /* Read/Update internal mouse position */
@@ -562,6 +591,17 @@ void Main_SetTitle(const char *title)
 		SDL_WM_SetCaption(PROG_NAME, "Hatari");
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Initialise emulation for some hardware components
+ * It is required to init those parts before parsing the parameters,
+ * (for example, we should init FDC before inserting a disk)
+ */
+static void Main_Init_HW(void)
+{
+	FDC_Init();
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * Initialise emulation
@@ -583,6 +623,13 @@ static void Main_Init(void)
 		fprintf(stderr, "Could not initialize the SDL library:\n %s\n", SDL_GetError() );
 		exit(-1);
 	}
+
+	if ( IPF_Init() != true )
+	{
+		fprintf(stderr, "Could not initialize the IPF support\n" );
+		exit(-1);
+	}
+
 	ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 	Resolution_Init();
 	SDLGui_Init();
@@ -599,6 +646,7 @@ static void Main_Init(void)
 
 	DSP_Init();
 	Floppy_Init();
+//	FDC_Init();
 	M68000_Init();                /* Init CPU emulation */
 	Audio_Init();
 	Keymap_Init();
@@ -662,6 +710,8 @@ static void Main_UnInit(void)
 	Screen_UnInit();
 	Exit680x0();
 
+	IPF_Exit();
+
 	/* SDL uninit: */
 	SDL_Quit();
 
@@ -720,7 +770,7 @@ static void Main_StatusbarSetup(void)
 		snprintf(message, sizeof(message), "Press %s for Options", keyname);
 		free(keyname);
 
-		Statusbar_AddMessage(message, 6000);
+		Statusbar_AddMessage(message, 5000);
 	}
 	/* update information loaded by Main_Init() */
 	Statusbar_UpdateInfo();
@@ -740,7 +790,10 @@ int main(int argc, char *argv[])
 	/* Initialize directory strings */
 	Paths_Init(argv[0]);
 
-	/* Set default configuration values: */
+	/* Init some HW components before parsing the configuration / parameters */
+	Main_Init_HW();
+
+	/* Set default configuration values */
 	Configuration_SetDefault();
 
 	/* Now load the values from the configuration file */
@@ -792,11 +845,11 @@ int main(int argc, char *argv[])
 	{
 		/* cleanly close the avi file */
 		Statusbar_AddMessage("Finishing AVI file...", 100);
-		Statusbar_Update(sdlscrn);
+		Statusbar_Update(sdlscrn, true);
 		Avi_StopRecording();
 	}
 	/* Un-init emulation system */
 	Main_UnInit();
 
-	return 0;
+	return nQuitValue;
 }
