@@ -169,7 +169,7 @@ Input -----/             |         ------------------------              |
     for long ones like MOVEM or DIV). In that case, we should not choose the highest priority interrupt
     among all the interrupts, but we should keep only the interrupts that chronologically happened first
     during this CPU instruction (and ignore the other interrupts' requests for this CPU instruction).
-  - When the MFP's main IRQ signal goes from 0 to 1, the signal is not immediatly visible to the CPU, but only
+  - When the MFP's main IRQ signal goes from 0 to 1, the signal is not immediately visible to the CPU, but only
     4 cycles later. This 4 cycle delay should be taken into account, depending at what time the signal
     went to 1 in the corresponding CPU instruction (the 4 cycle delay can be "included" in the CPU instruction
     in some cases)
@@ -777,6 +777,15 @@ static int MFP_StartTimer_AB(Uint8 TimerControl, Uint16 TimerData, interrupt_id
 			TimerData = 256;
 		TimerClockCycles = MFP_REG_TO_CYCLES ( TimerData, TimerControl );
 
+		/* [NP] FIXME : Temporary fix for Lethal Xcess calibration routine to remove top border : */
+		/* the routine expects that the delay is not always stable, there must be a small */
+		/* jitter due to the clock difference between CPU and MFP */
+		if ( ( M68000_GetPC() == 0x14d78 ) && ( STMemory_ReadLong ( 0x14d6c ) == 0x11faff75 ) )
+		{
+//			fprintf ( stderr , "mfp add jitter %d\n" , TimerClockCycles );
+			TimerClockCycles += rand()%5-2;		/* add jitter for wod2 */
+		}
+
 		if (LOG_TRACE_LEVEL(TRACE_MFP_START))
 		{
 			int FrameCycles, HblCounterVideo, LineCycles;
