@@ -92,11 +92,11 @@ const char MSA_fileid[] = "Hatari msa.c : " __DATE__ " " __TIME__;
 
 typedef struct
 {
-	short int ID;                 /* Word  ID marker, should be $0E0F */
-	short int SectorsPerTrack;    /* Word  Sectors per track */
-	short int Sides;              /* Word  Sides (0 or 1; add 1 to this to get correct number of sides) */
-	short int StartingTrack;      /* Word  Starting track (0-based) */
-	short int EndingTrack;        /* Word  Ending track (0-based) */
+	Uint16	ID;			/* Word : ID marker, should be $0E0F */
+	Uint16	SectorsPerTrack;	/* Word : Sectors per track */
+	Uint16	Sides;			/* Word : Sides (0 or 1; add 1 to this to get correct number of sides) */
+	Uint16	StartingTrack;		/* Word : Starting track (0-based) */
+	Uint16	EndingTrack;		/* Word : Ending track (0-based) */
 } MSAHEADERSTRUCT;
 
 #define MSA_WORKSPACE_SIZE  (1024*1024)  /* Size of workspace to use when saving MSA files */
@@ -149,7 +149,7 @@ Uint8 *MSA_UnCompress(Uint8 *pMSAFile, long *pImageSize)
 
 		/* Set pointers */
 		pImageBuffer = (Uint8 *)pBuffer;
-		pMSAImageBuffer = (Uint8 *)((unsigned long)pMSAFile + sizeof(MSAHEADERSTRUCT));
+		pMSAImageBuffer = pMSAFile + sizeof(MSAHEADERSTRUCT);
 
 		/* Uncompress to memory as '.ST' disk image - NOTE: assumes 512 bytes
 		 * per sector (use NUMBYTESPERSECTOR define)!!! */
@@ -161,7 +161,7 @@ Uint8 *MSA_UnCompress(Uint8 *pMSAFile, long *pImageSize)
 
 				/* Uncompress MSA Track, first check if is not compressed */
 				DataLength = do_get_mem_word(pMSAImageBuffer);
-				pMSAImageBuffer += sizeof(short int);
+				pMSAImageBuffer += sizeof(Uint16);
 				if (DataLength == nBytesPerTrack)
 				{
 					/* No compression on track, simply copy and continue */
@@ -191,7 +191,7 @@ Uint8 *MSA_UnCompress(Uint8 *pMSAFile, long *pImageSize)
 								fprintf(stderr, "MSA_UnCompress: Illegal run length -> corrupted disk image?\n");
 								RunLength = nBytesPerTrack - NumBytesUnCompressed;
 							}
-							pMSAImageBuffer += sizeof(short int);
+							pMSAImageBuffer += sizeof(Uint16);
 							for (i = 0; i < RunLength; i++)
 								*pImageBuffer++ = Data;   /* Copy byte */
 							NumBytesUnCompressed += RunLength;
@@ -202,7 +202,7 @@ Uint8 *MSA_UnCompress(Uint8 *pMSAFile, long *pImageSize)
 		}
 
 		/* Set size of loaded image */
-		*pImageSize = (unsigned long)pImageBuffer-(unsigned long)pBuffer;
+		*pImageSize = pImageBuffer-pBuffer;
 	}
 
 	/* Return pointer to buffer, NULL if failed */
@@ -215,7 +215,7 @@ Uint8 *MSA_UnCompress(Uint8 *pMSAFile, long *pImageSize)
  * Uncompress .MSA file into memory, set number bytes of the disk image and
  * return a pointer to the buffer.
  */
-Uint8 *MSA_ReadDisk(const char *pszFileName, long *pImageSize)
+Uint8 *MSA_ReadDisk(int Drive, const char *pszFileName, long *pImageSize, int *pImageType)
 {
 	Uint8 *pMsaFile;
 	Uint8 *pDiskBuffer = NULL;
@@ -233,6 +233,10 @@ Uint8 *MSA_ReadDisk(const char *pszFileName, long *pImageSize)
 		free(pMsaFile);
 	}
 
+	if ( ( pMsaFile == NULL ) || ( pDiskBuffer == NULL ) )
+		return NULL;
+
+	*pImageType = FLOPPY_IMAGE_TYPE_MSA;
 	/* Return pointer to buffer, NULL if failed */
 	return pDiskBuffer;
 }
@@ -286,12 +290,12 @@ static int MSA_FindRunOfBytes(Uint8 *pBuffer, int nBytesInBuffer)
 /**
  * Save compressed .MSA file from memory buffer. Returns true is all OK
  */
-bool MSA_WriteDisk(const char *pszFileName, Uint8 *pBuffer, int ImageSize)
+bool MSA_WriteDisk(int Drive, const char *pszFileName, Uint8 *pBuffer, int ImageSize)
 {
 #ifdef SAVE_TO_MSA_IMAGES
 
 	MSAHEADERSTRUCT *pMSAHeader;
-	unsigned short int *pMSADataLength;
+	Uint16 *pMSADataLength;
 	Uint8 *pMSAImageBuffer, *pMSABuffer, *pImageBuffer;
 	Uint16 nSectorsPerTrack, nSides, nCompressedBytes, nBytesPerTrack;
 	bool nRet;
