@@ -28,6 +28,7 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include "debugui.h"
 #include "file.h"
 #include "floppy.h"
+#include "fdc.h"
 #include "screen.h"
 #include "sound.h"
 #include "video.h"
@@ -39,11 +40,10 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include "avi_record.h"
 #include "hatari-glue.h"
 #include "68kDisass.h"
-
+#include "xbios.h"
 
 bool bLoadAutoSave;        /* Load autosave memory snapshot at startup */
 bool bLoadMemorySave;      /* Load memory snapshot provided via option at startup */
-bool bBiosIntercept;       /* whether UAE should intercept Bios & XBios calls */
 bool AviRecordOnStartup;   /* Start avi recording at startup */
 
 int ConOutDevice = CONOUT_DEVICE_NONE; /* device number for xconout device to track */
@@ -65,16 +65,17 @@ enum {
 	OPT_WINDOW,
 	OPT_GRAB,
 	OPT_FRAMESKIPS,
+	OPT_SLOWDOWN,
 	OPT_STATUSBAR,
 	OPT_DRIVE_LED,
+	OPT_MAXWIDTH,
+	OPT_MAXHEIGHT,
 	OPT_FORCEBPP,
 	OPT_BORDERS,		/* ST/STE display options */
 	OPT_RESOLUTION_ST,
 	OPT_SPEC512,
 	OPT_ZOOM,
-	OPT_RESOLUTION,		/* Falcon/TT display options */
-	OPT_MAXWIDTH,
-	OPT_MAXHEIGHT,
+	OPT_RESOLUTION,		/* TT/Falcon display options */
 	OPT_FORCE_MAX,
 	OPT_ASPECT,
 	OPT_VDI,		/* VDI options */
@@ -98,7 +99,11 @@ enum {
 	OPT_MIDI_OUT,
 	OPT_RS232_IN,
 	OPT_RS232_OUT,
-	OPT_DISKA,		/* disk options */
+	OPT_DRIVEA,		/* disk options */
+	OPT_DRIVEB,
+	OPT_DRIVEA_HEADS,
+	OPT_DRIVEB_HEADS,
+	OPT_DISKA,
 	OPT_DISKB,
 	OPT_SLOWFLOPPY,
 	OPT_FASTFLOPPY,
@@ -139,6 +144,7 @@ enum {
 	OPT_WINCON,		/* debug options */
 #endif
 	OPT_DEBUG,
+	OPT_EXCEPTIONS,
 	OPT_BIOSINTERCEPT,
 	OPT_CONOUT,
 	OPT_DISASM,
@@ -190,15 +196,21 @@ static const opt_t HatariOptions[] = {
 	{ OPT_FULLSCREEN,"-f", "--fullscreen",
 	  NULL, "Start emulator in fullscreen mode" },
 	{ OPT_WINDOW,    "-w", "--window",
-	  NULL, "Start emulator in window mode" },
+	  NULL, "Start emulator in windowed mode" },
 	{ OPT_GRAB, NULL, "--grab",
-	  NULL, "Grab mouse (also) in window mode" },
+	  NULL, "Grab mouse (also) in windowed mode" },
 	{ OPT_FRAMESKIPS, NULL, "--frameskips",
 	  "<x>", "Skip <x> frames after each shown frame (0=off, >4=auto/max)" },
+	{ OPT_SLOWDOWN, NULL, "--slowdown",
+	  "<x>", "VBL wait time multiplier (1-8, default 1)" },
 	{ OPT_STATUSBAR, NULL, "--statusbar",
 	  "<bool>", "Show statusbar (floppy leds etc)" },
 	{ OPT_DRIVE_LED,   NULL, "--drive-led",
 	  "<bool>", "Show overlay drive led when statusbar isn't shown" },
+	{ OPT_MAXWIDTH, NULL, "--max-width",
+	  "<x>", "Maximum window width for borders & zooming" },
+	{ OPT_MAXHEIGHT, NULL, "--max-height",
+	  "<x>", "Maximum window height for borders & zooming" },
 	{ OPT_FORCEBPP, NULL, "--bpp",
 	  "<x>", "Force internal bitdepth (x = 8/15/16/32, 0=disable)" },
 
@@ -206,19 +218,15 @@ static const opt_t HatariOptions[] = {
 	{ OPT_BORDERS, NULL, "--borders",
 	  "<bool>", "Show screen borders (for overscan demos etc)" },
 	{ OPT_RESOLUTION_ST, NULL, "--desktop-st",
-	  "<bool>", "Keep desktop resolution on fullscreen (no zoom)" },
+	  "<bool>", "Keep desktop resolution on fullscreen" },
 	{ OPT_SPEC512, NULL, "--spec512",
 	  "<x>", "Spec512 palette threshold (0 <= x <= 512, 0=disable)" },
 	{ OPT_ZOOM, "-z", "--zoom",
 	  "<x>", "Double small resolutions (1=no, 2=yes)" },
 
-	{ OPT_HEADER, NULL, NULL, NULL, "Falcon/TT specific display" },
+	{ OPT_HEADER, NULL, NULL, NULL, "TT/Falcon specific display" },
 	{ OPT_RESOLUTION, NULL, "--desktop",
 	  "<bool>", "Keep desktop resolution on fullscreen" },
-	{ OPT_MAXWIDTH, NULL, "--max-width",
-	  "<x>", "Maximum window width for zooming" },
-	{ OPT_MAXHEIGHT, NULL, "--max-height",
-	  "<x>", "Maximum window height for zooming" },
 	{ OPT_FORCE_MAX, NULL, "--force-max",
 	  "<bool>", "Resolution fixed to given max values" },
 	{ OPT_ASPECT, NULL, "--aspect",
@@ -278,6 +286,14 @@ static const opt_t HatariOptions[] = {
 	  "<file>", "Enable serial port and use <file> as the output device" },
 	
 	{ OPT_HEADER, NULL, NULL, NULL, "Disk" },
+	{ OPT_DRIVEA, NULL, "--drive-a",
+	  "<bool>", "Enable/disable drive A (default is on)" },
+	{ OPT_DRIVEB, NULL, "--drive-b",
+	  "<bool>", "Enable/disable drive B (default is on)" },
+	{ OPT_DRIVEA_HEADS, NULL, "--drive-a-heads",
+	  "<x>", "Set number of heads for drive A (1=single sided, 2=double sided)" },
+	{ OPT_DRIVEB_HEADS, NULL, "--drive-b-heads",
+	  "<x>", "Set number of heads for drive B (1=single sided, 2=double sided)" },
 	{ OPT_DISKA, NULL, "--disk-a",
 	  "<file>", "Set disk image for floppy drive A" },
 	{ OPT_DISKB, NULL, "--disk-b",
@@ -370,8 +386,10 @@ static const opt_t HatariOptions[] = {
 #endif
 	{ OPT_DEBUG,     "-D", "--debug",
 	  NULL, "Toggle whether CPU exceptions invoke debugger" },
+	{ OPT_EXCEPTIONS, NULL, "--debug-except",
+	  "<flags>", "Exceptions invoking debugger, see '--debug-except help'" },
 	{ OPT_BIOSINTERCEPT, NULL, "--bios-intercept",
-	  NULL, "Toggle X/Bios interception & Hatari XBios 255 support" },
+	  NULL, "Toggle XBios command parsing support" },
 	{ OPT_CONOUT,   NULL, "--conout",
 	  "<device>", "Show console output (0-7, 2=VT-52 terminal)" },
 	{ OPT_DISASM,   NULL, "--disasm",
@@ -379,7 +397,7 @@ static const opt_t HatariOptions[] = {
 	{ OPT_NATFEATS, NULL, "--natfeats",
 	  "<bool>", "Whether Native Features support is enabled" },
 	{ OPT_TRACE,   NULL, "--trace",
-	  "<trace1,...>", "Activate emulation tracing, see '--trace help'" },
+	  "<flags>", "Activate emulation tracing, see '--trace help'" },
 	{ OPT_TRACEFILE, NULL, "--trace-file",
 	  "<file>", "Save trace output to <file> (default=stderr)" },
 	{ OPT_PARSE, NULL, "--parse",
@@ -603,7 +621,7 @@ static bool Opt_Bool(const char *arg, int optid, bool *conf)
 	str = input;
 	while (*str)
 	{
-		*str++ = tolower(*arg++);
+		*str++ = tolower((unsigned char)*arg++);
 	}
 	for (bool_str = enablers; *bool_str; bool_str++)
 	{
@@ -971,6 +989,13 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.Screen.nFrameSkips = skips;
 			break;
 
+		case OPT_SLOWDOWN:
+			if (!Main_SetVBLSlowdown(atoi(argv[++i])))
+			{
+				return Opt_ShowError(OPT_SLOWDOWN, argv[i], "Invalid VBL wait slowdown multiplier");
+			}
+			break;
+
 		case OPT_STATUSBAR:
 			ok = Opt_Bool(argv[++i], OPT_STATUSBAR, &ConfigureParams.Screen.bShowStatusbar);
 			break;
@@ -1031,8 +1056,8 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			else
 			{
-				ConfigureParams.Screen.nMaxWidth = 320;
-				ConfigureParams.Screen.nMaxHeight = 200;
+				ConfigureParams.Screen.nMaxWidth = 1*(48+320+48);
+				ConfigureParams.Screen.nMaxHeight = 1*NUM_VISIBLE_LINES+12;
 			}
 			break;
 
@@ -1215,6 +1240,32 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 
 			/* disk options */
+		case OPT_DRIVEA:
+			ok = Opt_Bool(argv[++i], OPT_DRIVEA, &ConfigureParams.DiskImage.EnableDriveA);
+			break;
+
+		case OPT_DRIVEB:
+			ok = Opt_Bool(argv[++i], OPT_DRIVEB, &ConfigureParams.DiskImage.EnableDriveB);
+			break;
+
+		case OPT_DRIVEA_HEADS:
+			val = atoi(argv[++i]);
+			if(val != 1 && val != 2)
+			{
+				return Opt_ShowError(OPT_DRIVEA_HEADS, argv[i], "Invalid number of heads");
+			}
+			ConfigureParams.DiskImage.DriveA_NumberOfHeads = val;
+			break;
+
+		case OPT_DRIVEB_HEADS:
+			val = atoi(argv[++i]);
+			if(val != 1 && val != 2)
+			{
+				return Opt_ShowError(OPT_DRIVEB_HEADS, argv[i], "Invalid number of heads");
+			}
+			ConfigureParams.DiskImage.DriveB_NumberOfHeads = val;
+			break;
+
 		case OPT_DISKA:
 			i += 1;
 			if (Floppy_SetDiskFileName(0, argv[i], NULL))
@@ -1299,9 +1350,9 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_ACSIHDIMAGE:
 			i += 1;
-			ok = Opt_StrCpy(OPT_ACSIHDIMAGE, true, ConfigureParams.HardDisk.szHardDiskImage,
-					argv[i], sizeof(ConfigureParams.HardDisk.szHardDiskImage),
-					&ConfigureParams.HardDisk.bUseHardDiskImage);
+			ok = Opt_StrCpy(OPT_ACSIHDIMAGE, true, ConfigureParams.Acsi[0].sDeviceFile,
+					argv[i], sizeof(ConfigureParams.Acsi[0].sDeviceFile),
+					&ConfigureParams.Acsi[0].bUseDevice);
 			if (ok)
 			{
 				bLoadAutoSave = false;
@@ -1629,29 +1680,42 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 #endif
 		case OPT_DEBUG:
-			if (bExceptionDebugging)
+			if (ExceptionDebugMask)
 			{
 				fprintf(stderr, "Exception debugging disabled.\n");
-				bExceptionDebugging = false;
+				ExceptionDebugMask = EXCEPT_NONE;
 			}
 			else
 			{
-				fprintf(stderr, "Exception debugging enabled.\n");
-				bExceptionDebugging = true;
+				ExceptionDebugMask = ConfigureParams.Log.nExceptionDebugMask;
+				fprintf(stderr, "Exception debugging enabled (0x%x).\n", ExceptionDebugMask);
+			}
+			break;
+
+		case OPT_EXCEPTIONS:
+			i += 1;
+			/* sets ConfigureParams.Log.nExceptionDebugMask */
+			errstr = Log_SetExceptionDebugMask(argv[i]);
+			if (errstr)
+			{
+				if (!errstr[0]) {
+					/* silent parsing termination */
+					return false;
+				}
+				return Opt_ShowError(OPT_EXCEPTIONS, argv[i], errstr);
+			}
+			if (ExceptionDebugMask)
+			{
+				/* already enabled, change run-time config */
+				int oldmask = ExceptionDebugMask;
+				ExceptionDebugMask = ConfigureParams.Log.nExceptionDebugMask;
+				fprintf(stderr, "Exception debugging changed (0x%x -> 0x%x).\n",
+					oldmask, ExceptionDebugMask);
 			}
 			break;
 
 		case OPT_BIOSINTERCEPT:
-			if (bBiosIntercept)
-			{
-				fprintf(stderr, "X/Bios interception disabled.\n");
-				bBiosIntercept = false;
-			}
-			else
-			{
-				fprintf(stderr, "X/Bios interception enabled.\n");
-				bBiosIntercept = true;
-			}
+			XBios_ToggleCommands();
 			break;
 
 		case OPT_CONOUT:
