@@ -127,6 +127,7 @@ const char PSG_fileid[] = "Hatari psg.c : " __DATE__ " " __TIME__;
 #include "video.h"
 #include "statusbar.h"
 #include "mfp.h"
+#include "fdc.h"
 
 
 Uint8 PSGRegisterSelect;        /* Write to 0xff8800 sets the register number used in read/write accesses */
@@ -255,6 +256,8 @@ Uint8 PSG_Get_DataRegister(void)
  */
 void PSG_Set_DataRegister(Uint8 val)
 {
+	Uint8	val_old;
+
 	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
 	{
 		int FrameCycles, HblCounterVideo, LineCycles;
@@ -275,6 +278,9 @@ void PSG_Set_DataRegister(Uint8 val)
 	/* the non masked value. */
 	PSGRegisterReadData = val;			/* store non masked value for PSG_Get_DataRegister */
 
+	/* Read previous content */
+	val_old = PSGRegisters[PSGRegisterSelect];
+
 	/* Copy value to PSGRegisters[] */
 	PSGRegisters[PSGRegisterSelect] = val;
 
@@ -342,6 +348,9 @@ void PSG_Set_DataRegister(Uint8 val)
 			Statusbar_SetFloppyLed(DRIVE_LED_B, LED_STATE_OFF);
 		}
 
+		/* Report a possible drive/side change */
+		FDC_SetDriveSide ( val_old & 7 , PSGRegisters[PSG_REG_IO_PORTA] & 7 );
+
 		/* Bit 3 - Centronics as input */
 		if(PSGRegisters[PSG_REG_IO_PORTA]&(1<<3))
 		{
@@ -374,7 +383,7 @@ void PSG_Set_DataRegister(Uint8 val)
 				/* FIXME: add code to handle IDE reset */
 			}
 		}
-	
+
 	}
 }
 
@@ -542,3 +551,16 @@ void PSG_ff8803_WriteByte(void)
 		}
 	}
 }
+
+
+/* ------------------------------------------------------------------
+ * YM-2149 register content dump (for debugger info command)
+ */
+void PSG_Info(Uint32 dummy)
+{
+	int i;
+	for(i = 0; i < ARRAYSIZE(PSGRegisters); i++)
+	{
+		fprintf(stderr, "Reg $%02X : $%02X\n", i, PSGRegisters[i]);
+	}
+}
