@@ -29,10 +29,9 @@ const char ScanDir_fileid[] = "Hatari scandir.c : " __DATE__ " " __TIME__;
  * Alphabetic order comparison routine.
  */
 #if !HAVE_ALPHASORT
-int alphasort(const void *d1, const void *d2)
+int alphasort(const struct dirent **d1, const struct dirent **d2)
 {
-	return strcmp((*(struct dirent * const *)d1)->d_name,
-		      (*(struct dirent * const *)d2)->d_name);
+	return strcmp((*d1)->d_name, (*d2)->d_name);
 }
 #endif
 
@@ -55,7 +54,9 @@ int alphasort(const void *d1, const void *d2)
  * Scan a directory for all its entries
  * Return -1 on error, number of entries on success
  */
-int scandir(const char *dirname, struct dirent ***namelist, int (*sdfilter)(struct dirent *), int (*dcomp)(const void *, const void *))
+int scandir(const char *dirname, struct dirent ***namelist,
+            int (*sdfilter)(const struct dirent *),
+            int (*dcomp)(const struct dirent **, const struct dirent **))
 {
 	struct dirent *d, *p = NULL, **names = NULL;
 	struct stat stb;
@@ -125,7 +126,8 @@ int scandir(const char *dirname, struct dirent ***namelist, int (*sdfilter)(stru
 	closedir(dirp);
 
 	if (nitems && dcomp != NULL)
-		qsort(names, nitems, sizeof(struct dirent *), dcomp);
+		qsort(names, nitems, sizeof(struct dirent *),
+		      (int (*)(const void *, const void *))dcomp);
 
 	*namelist = names;
 
@@ -161,16 +163,18 @@ error_out:
 /**
  * Alphabetic order comparison routine.
  */
-int alphasort(const void *d1, const void *d2)
+int alphasort(const struct dirent **d1, const struct dirent **d2)
 {
-	return stricmp((*(struct dirent * const *)d1)->d_name, (*(struct dirent * const *)d2)->d_name);
+	return stricmp((*d1)->d_name, (*d2)->d_name);
 }
 
 /*-----------------------------------------------------------------------*/
 /**
  * Scan a directory for all its entries
  */
-int scandir(const char *dirname, struct dirent ***namelist, int (*sdfilter)(struct dirent *), int (*dcomp)(const void *, const void *))
+int scandir(const char *dirname, struct dirent ***namelist,
+            int (*sdfilter)(const struct dirent *),
+            int (*dcomp)(const struct dirent **, const struct dirent **))
 {
 	int len;
 	char *findIn, *d;
@@ -294,7 +298,8 @@ int scandir(const char *dirname, struct dirent ***namelist, int (*sdfilter)(stru
 #endif
 
 	if (dcomp)
-		qsort (dir, nDir, sizeof(*dir),dcomp);
+		qsort(dir, nDir, sizeof(*dir),
+		      (int (*)(const void *, const void *))dcomp);
 
 	*namelist = dir;
 	return nDir;
