@@ -47,6 +47,13 @@ const char Screen_fileid[] = "Hatari screen.c : " __DATE__ " " __TIME__;
 #include "falcon/videl.h"
 #include "falcon/hostscreen.h"
 
+#define DEBUG 0
+
+#if DEBUG
+# define DEBUGPRINT(x) printf x
+#else
+# define DEBUGPRINT(x)
+#endif
 
 /* extern for several purposes */
 SDL_Surface *sdlscrn = NULL;                /* The SDL screen surface */
@@ -411,8 +418,13 @@ static void Screen_SetResolution(void)
 			int leftY = maxH - (Height + Statusbar_GetHeightForSize(Width, Height));
 
 			Screen_SetBorderPixels(leftX/nZoom, leftY/nZoom);
+			DEBUGPRINT(("resolution limit:\n\t%d x %d\nlimited resolution:\n\t", maxW, maxH));
+			DEBUGPRINT(("%d * (%d + %d + %d) x (%d + %d + %d)\n", nZoom,
+				    nBorderPixelsLeft, Width/nZoom, nBorderPixelsRight,
+				    nBorderPixelsTop, Height/nZoom, nBorderPixelsBottom));
 			Width += (nBorderPixelsRight + nBorderPixelsLeft)*nZoom;
 			Height += (nBorderPixelsTop + nBorderPixelsBottom)*nZoom;
+			DEBUGPRINT(("\t= %d x %d (+ statusbar)\n", Width, Height));
 		}
 	}
 	
@@ -438,7 +450,7 @@ static void Screen_SetResolution(void)
 			Height = maxH;
 			Width = maxW;
 		}
-		sdlVideoFlags  = SDL_HWSURFACE|SDL_FULLSCREEN|SDL_HWPALETTE/*|SDL_DOUBLEBUF*/;
+		sdlVideoFlags  = SDL_HWSURFACE|SDL_FULLSCREEN/*|SDL_DOUBLEBUF*/;
 		/* SDL_DOUBLEBUF helps avoiding tearing and can be faster on suitable HW,
 		 * but it doesn't work with partial screen updates done by the ST screen
 		 * update code or the Hatari GUI, so double buffering is disabled.
@@ -446,7 +458,11 @@ static void Screen_SetResolution(void)
 	}
 	else
 	{
-		sdlVideoFlags  = SDL_SWSURFACE|SDL_HWPALETTE;
+		sdlVideoFlags  = SDL_SWSURFACE;
+	}
+	if (BitCount <= 8)
+	{
+		sdlVideoFlags |= SDL_HWPALETTE;
 	}
 
 	/* Check if we really have to change the video mode: */
@@ -466,9 +482,9 @@ static void Screen_SetResolution(void)
 		}
 		
 		/* Set new video mode */
-		//fprintf(stderr,"Requesting video mode %i %i %i\n", Width, Height, BitCount);
+		DEBUGPRINT(("SDL screen request: %d x %d @ %d (%s)\n", Width, Height, BitCount, bInFullScreen?"fullscreen":"windowed"));
 		sdlscrn = SDL_SetVideoMode(Width, Height, BitCount, sdlVideoFlags);
-		//fprintf(stderr,"Got video mode %i %i %i\n", sdlscrn->w, sdlscrn->h, sdlscrn->format->BitsPerPixel);
+		DEBUGPRINT(("SDL screen granted: %d x %d @ %d\n", sdlscrn->w, sdlscrn->h, sdlscrn->format->BitsPerPixel));
 
 		/* By default ConfigureParams.Screen.nForceBpp and therefore
 		 * BitCount is zero which means "SDL color depth autodetection".
@@ -1070,7 +1086,7 @@ static void Screen_UnLock(void)
 /**
  * Blit our converted ST screen to window/full-screen
  */
-static void Screen_Blit(void)
+static void Screen_Blit(SDL_Rect *sbar_rect)
 {
 	unsigned char *pTmpScreen;
 
@@ -1089,7 +1105,15 @@ static void Screen_Blit(void)
 # endif
 #endif
 	{
-		SDL_UpdateRects(sdlscrn, 1, &STScreenRect);
+		int count = 1;
+		SDL_Rect rects[2];
+		rects[0] = STScreenRect;
+		if (sbar_rect)
+		{
+			rects[1] = *sbar_rect;
+			count = 2;
+		}
+		SDL_UpdateRects(sdlscrn, count, rects);
 	}
 
 	/* Swap copy/raster buffers in screen. */
@@ -1110,6 +1134,7 @@ static bool Screen_DrawFrame(bool bForceFlip)
 	int new_res;
 	void (*pDrawFunction)(void);
 	static bool bPrevFrameWasSpec512 = false;
+	SDL_Rect *sbar_rect;
 
 	/* Scan palette/resolution masks for each line and build up palette/difference tables */
 	new_res = Screen_ComparePaletteMask(STRes);
@@ -1125,8 +1150,8 @@ static bool Screen_DrawFrame(bool bForceFlip)
 	 * and saved by Statusbar_OverlayBackup()
 	 */
 	Statusbar_OverlayRestore(sdlscrn);
-	
-	/* Lock screen ready for drawing */
+
+	/* Lock screen for direct screen surface format writes */
 	if (Screen_Lock())
 	{
 		bScreenContentsChanged = false;      /* Did change (ie needs blit?) */
@@ -1180,18 +1205,18 @@ static bool Screen_DrawFrame(bool bForceFlip)
 		/* Unlock screen */
 		Screen_UnLock();
 
-		/* draw statusbar or overlay led(s) after unlock */
+		/* draw overlay led(s) or statusbar after unlock */
 		Statusbar_OverlayBackup(sdlscrn);
-		Statusbar_Update(sdlscrn);
+		sbar_rect = Statusbar_Update(sdlscrn, false);
 		
 		/* Clear flags, remember type of overscan as if change need screen full update */
 		pFrameBuffer->bFullUpdate = false;
 		pFrameBuffer->OverscanModeCopy = OverscanMode;
 
 		/* And show to user */
-		if (bScreenContentsChanged || bForceFlip)
+		if (bScreenContentsChanged || bForceFlip || sbar_rect)
 		{
-			Screen_Blit();
+			Screen_Blit(sbar_rect);
 		}
 
 		return bScreenContentsChanged;
