@@ -222,10 +222,16 @@ static void ShortCut_InsertDisk(int drive)
 {
 	char *selname, *zip_path = NULL;
 	const char *tmpname;
+	char FileNameB[ FILENAME_MAX ];
+	char ZipPathB[ FILENAME_MAX ];
 
 	if (SDLGui_SetScreen(sdlscrn))
 		return;
 
+	/* Save current names for drive 1 before checking autoinsert */
+	strcpy ( FileNameB , ConfigureParams.DiskImage.szDiskFileName[ 1 ] );
+	strcpy ( ZipPathB , ConfigureParams.DiskImage.szDiskZipPath[ 1 ] );
+
 	if (ConfigureParams.DiskImage.szDiskFileName[drive][0])
 		tmpname = ConfigureParams.DiskImage.szDiskFileName[drive];
 	else
@@ -245,6 +251,12 @@ static void ShortCut_InsertDisk(int drive)
 		free(selname);
 		
 		Floppy_InsertDiskIntoDrive(0);
+
+		/* Check if inserting into drive 0 also changed drive 1 with autoinsert */
+		if ( ( strcmp ( FileNameB , ConfigureParams.DiskImage.szDiskFileName[ 1 ] ) != 0 )
+		  || ( strcmp ( FileNameB , ConfigureParams.DiskImage.szDiskZipPath[ 1 ] ) != 0 ) )
+			Floppy_InsertDiskIntoDrive(1);
+
 	}
 	Main_UnPauseEmulation();
 }
@@ -306,7 +318,7 @@ void ShortCut_ActKey(void)
 		ShortCut_Pause();              /* Invoke Pause */
 		break;
 	 case SHORTCUT_QUIT:
-		Main_RequestQuit();
+		Main_RequestQuit(0);
 		break;
 	 case SHORTCUT_LOADMEM:
 		MemorySnapShot_Restore(ConfigureParams.Memory.szMemoryCaptureFileName, true);
