@@ -19,6 +19,8 @@ const char ZIP_fileid[] = "Hatari zip.c : " __DATE__ " " __TIME__;
 #include "dim.h"
 #include "file.h"
 #include "floppy.h"
+#include "floppy_ipf.h"
+#include "floppy_stx.h"
 #include "log.h"
 #include "msa.h"
 #include "st.h"
@@ -35,18 +37,19 @@ const char ZIP_fileid[] = "Hatari zip.c : " __DATE__ " " __TIME__;
 
 #define ZIP_PATH_MAX  256
 
-#define ZIP_FILE_ST   1
-#define ZIP_FILE_MSA  2
-#define ZIP_FILE_DIM  3
-
+#if HAVE_LIBZ
 
 /* Possible disk image extensions to scan for */
 static const char * const pszDiskNameExts[] =
 {
-  ".msa",
-  ".st",
-  ".dim",
-  NULL
+	".msa",
+	".st",
+	".dim",
+	".ipf",
+	".raw",
+	".ctr",
+	".stx",
+	NULL
 };
 
 
@@ -313,6 +316,7 @@ struct dirent **ZIP_GetFilesDir(const zip_dir *zip, const char *dir, int *entrie
 		{
 			perror("ZIP_GetFilesDir");
 			ZIP_FreeFentries(fentries, i+1);
+			ZIP_FreeZipDir(files);
 			return NULL;
 		}
 		strcpy(fentries[i]->d_name, files->names[i]);
@@ -328,7 +332,7 @@ struct dirent **ZIP_GetFilesDir(const zip_dir *zip, const char *dir, int *entrie
 /**
  * Check an image file in the archive, return the uncompressed length
  */
-static long ZIP_CheckImageFile(unzFile uf, char *filename, int namelen, int *pDiskType)
+static long ZIP_CheckImageFile(unzFile uf, char *filename, int namelen, int *pImageType)
 {
 	unz_file_info file_info;
 
@@ -344,26 +348,38 @@ static long ZIP_CheckImageFile(unzFile uf, char *filename, int namelen, int *pDi
 		return -1;
 	}
 
-	/* check for a .msa or .st extension */
+	/* check for .stx, .ipf, .msa, .dim or .st extension */
+	if (STX_FileNameIsSTX(filename, false))
+	{
+		*pImageType = FLOPPY_IMAGE_TYPE_STX;
+		return file_info.uncompressed_size;
+	}
+
+	if (IPF_FileNameIsIPF(filename, false))
+	{
+		*pImageType = FLOPPY_IMAGE_TYPE_IPF;
+		return file_info.uncompressed_size;
+	}
+
 	if (MSA_FileNameIsMSA(filename, false))
 	{
-		*pDiskType = ZIP_FILE_MSA;
+		*pImageType = FLOPPY_IMAGE_TYPE_MSA;
 		return file_info.uncompressed_size;
 	}
 
 	if (ST_FileNameIsST(filename, false))
 	{
-		*pDiskType = ZIP_FILE_ST;
+		*pImageType = FLOPPY_IMAGE_TYPE_ST;
 		return file_info.uncompressed_size;
 	}
 
 	if (DIM_FileNameIsDIM(filename, false))
 	{
-		*pDiskType = ZIP_FILE_DIM;
+		*pImageType = FLOPPY_IMAGE_TYPE_DIM;
 		return file_info.uncompressed_size;
 	}
 
-	Log_Printf(LOG_ERROR, "Not an .ST, .MSA or .DIM file.\n");
+	Log_Printf(LOG_ERROR, "Not an .ST, .MSA, .DIM, .IPF or .STX file.\n");
 	return 0;
 }
 
@@ -386,6 +402,7 @@ static char *ZIP_FirstFile(const char *filename, const char * const ppsExts[])
 	if (!name)
 	{
 		perror("ZIP_FirstFile");
+		ZIP_FreeZipDir(files);
 		return NULL;
 	}
 
@@ -415,7 +432,11 @@ static char *ZIP_FirstFile(const char *filename, const char * const ppsExts[])
 	ZIP_FreeZipDir(files);
 
 	if (name[0] == '\0')
+	{
+		free(name);
 		return NULL;
+	}
+
 	return name;
 }
 
@@ -485,16 +506,16 @@ static void *ZIP_ExtractFile(unzFile uf, const char *filename, uLong size)
  * Load disk image from a .ZIP archive into memory, set  the number
  * of bytes loaded into pImageSize and return the data or NULL on error.
  */
-Uint8 *ZIP_ReadDisk(const char *pszFileName, const char *pszZipPath, long *pImageSize)
+Uint8 *ZIP_ReadDisk(int Drive, const char *pszFileName, const char *pszZipPath, long *pImageSize, int *pImageType)
 {
 	uLong ImageSize=0;
 	unzFile uf=NULL;
 	Uint8 *buf;
 	char *path;
-	int nDiskType = -1;
 	Uint8 *pDiskBuffer = NULL;
 
 	*pImageSize = 0;
+	*pImageType = FLOPPY_IMAGE_TYPE_NONE;
 
 	uf = unzOpen(pszFileName);
 	if (uf == NULL)
@@ -526,7 +547,7 @@ Uint8 *ZIP_ReadDisk(const char *pszFileName, const char *pszZipPath, long *pImag
 		path[ZIP_PATH_MAX-1] = '\0';
 	}
 
-	ImageSize = ZIP_CheckImageFile(uf, path, ZIP_PATH_MAX, &nDiskType);
+	ImageSize = ZIP_CheckImageFile(uf, path, ZIP_PATH_MAX, pImageType);
 	if (ImageSize <= 0)
 	{
 		unzClose(uf);
@@ -547,19 +568,34 @@ Uint8 *ZIP_ReadDisk(const char *pszFileName, const char *pszZipPath, long *pImag
 		return NULL;  /* failed extraction, return error */
 	}
 
-	switch(nDiskType) {
-	case ZIP_FILE_MSA:
+	switch(*pImageType) {
+	case FLOPPY_IMAGE_TYPE_IPF:
+#ifndef HAVE_CAPSIMAGE
+		Log_AlertDlg(LOG_ERROR, "This version of Hatari was not built with IPF support, this disk image can't be handled.");
+		return NULL;
+#else
+		/* return buffer */
+		pDiskBuffer = buf;
+		break;
+#endif
+	case FLOPPY_IMAGE_TYPE_STX:
+		/* return buffer */
+		pDiskBuffer = buf;
+		break;
+	case FLOPPY_IMAGE_TYPE_MSA:
 		/* uncompress the MSA file */
 		pDiskBuffer = MSA_UnCompress(buf, (long *)&ImageSize);
 		free(buf);
 		buf = NULL;
 		break;
-	case ZIP_FILE_DIM:
+	case FLOPPY_IMAGE_TYPE_DIM:
 		/* Skip DIM header */
 		ImageSize -= 32;
 		memmove(buf, buf+32, ImageSize);
-		/* ...and passthrough */
-	case ZIP_FILE_ST:
+		/* return buffer */
+		pDiskBuffer = buf;
+		break;
+	case FLOPPY_IMAGE_TYPE_ST:
 		/* ST image => return buffer directly */
 		pDiskBuffer = buf;
 		break;
@@ -573,18 +609,6 @@ Uint8 *ZIP_ReadDisk(const char *pszFileName, const char *pszZipPath, long *pImag
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Save .ZIP file from memory buffer. Returns true if all is OK.
- *
- * Not yet implemented.
- */
-bool ZIP_WriteDisk(const char *pszFileName,unsigned char *pBuffer,int ImageSize)
-{
-	return false;
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Load first file from a .ZIP archive into memory, and return the number
@@ -645,3 +669,37 @@ Uint8 *ZIP_ReadFirstFile(const char *pszFileName, long *pImageSize, const char *
 
 	return pBuffer;
 }
+
+#else
+
+bool ZIP_FileNameIsZIP(const char *pszFileName)
+{
+	return false;
+}
+Uint8 *ZIP_ReadDisk(int Drive, const char *name, const char *path, long *size)
+{
+	return NULL;
+}
+struct dirent **ZIP_GetFilesDir(const zip_dir *zip, const char *dir, int *entries)
+{
+	return NULL;
+}
+zip_dir *ZIP_GetFiles(const char *pszFileName)
+{
+	return NULL;
+}
+void ZIP_FreeZipDir(zip_dir *f_zd)
+{
+}
+
+#endif  /* HAVE_LIBZ */
+
+/**
+ * Save .ZIP file from memory buffer. Returns true if all is OK.
+ *
+ * Not yet implemented.
+ */
+bool ZIP_WriteDisk(int Drive, const char *pszFileName,unsigned char *pBuffer,int ImageSize)
+{
+	return false;
+}
